/*
 * Copyright 2003 Federal Chancellery Austria
 * MOA-ID has been developed in a cooperation between BRZ, the Federal
 * Chancellery Austria - ICT staff unit, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * http://www.osor.eu/eupl/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 */


package at.gv.egovernment.moaspss.logging;

/**
 * Encapsulates contextual information (i.e. per request information) for
 * logging purposes.
 * 
 * @author Patrick Peck
 * @version $Id$
 */
public class LoggingContext {
  /** The name of the node ID system property. */
  public static final String NODE_ID_PROPERTY = "moa.node-id";
  
  /** The current transaction ID. */
  private String transactionID;
  /** The node ID. */
  private String nodeID;
  
  /**
   * Create a new <code>LoggingContext</code>.
   * 
   * @param transactionID The transaction ID. May be <code>null</code>.
   */
  public LoggingContext(String transactionID) {
    this.transactionID = transactionID;
    this.nodeID = System.getProperty(NODE_ID_PROPERTY);
  }
  
  /**
   * Return the transaction ID.
   * 
   * @return The transaction ID.
   */
  public String getTransactionID() {
    return transactionID;
  }
  
  /**
   * Return the node ID.
   * 
   * @return The node ID.
   */
  public String getNodeID() {
    return nodeID;
  }
}
