/*
 * Copyright 2003 Federal Chancellery Austria
 * MOA-ID has been developed in a cooperation between BRZ, the Federal
 * Chancellery Austria - ICT staff unit, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * http://www.osor.eu/eupl/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 */


package at.gv.egovernment.moaspss.logging;

/**
 * Provides each thread with a single instance of <code>LoggingContext</code>.
 * 
 * @author Patrick Peck
 * @version $Id$
 */
public class LoggingContextManager {
  /** The single instance of this class. */
  private static LoggingContextManager instance = null;
  
  /** The <code>LoggingContext</code> for each thread. */
  private ThreadLocal context;
  
  /**
   * Get the single instance of the <code>LoggingContextManager</code> class.
   * 
   * @return LoggingContextManager The single instance.
   */
  public static synchronized LoggingContextManager getInstance() {
    if (instance == null) {
      instance = new LoggingContextManager();
    }
    return instance;
  }
  
  /**
   * Creates a new <code>LoggingContextManager</code>.
   * 
   * Protected to disallow direct instantiation.
   */
  protected LoggingContextManager() {
    context = new ThreadLocal();
  }
  
  /**
   * Set the <code>LoggingContext</code> context for the current thread.
   * 
   * @param ctx The <code>LoggingContext</code> for the current thread.
   */
  public void setLoggingContext(LoggingContext ctx) {
    context.set(ctx);
  }
  
  /**
   * Return the <code>LoggingContext</code> for the current thread.
   * 
   * @return LoggingContext The <code>LoggingContext</code> for the current
   * thread, or <code>null</code> if none has been set.
   */
  public LoggingContext getLoggingContext() {
    return (LoggingContext) context.get();
  }

}
