/*
 * Copyright 2003 Federal Chancellery Austria
 * MOA-ID has been developed in a cooperation between BRZ, the Federal
 * Chancellery Austria - ICT staff unit, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * http://www.osor.eu/eupl/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 */


package at.gv.egovernment.moaspss.util;

import java.util.Iterator;
import java.util.List;

/**
 * Various utility methods for dealing with <code>java.util.Collection</code> 
 * classes. 
 * 
 * @author Patrick Peck
 * @version $Id$
 */
public class CollectionUtils {
  
  /**
   * Convert a <code>List</code> of <code>Number</code> objects to an 
   * <code>int</code> array.
   * 
   * @param nums The <code>List</code> containing the numbers whose integer
   * value to put into the result. 
   * @return The <code>int</code> values of the <code>Number</code>s contained
   * in <code>nums</code>.
   */
  public static int[] toIntArray(List nums) {
    int[] result = new int[nums.size()];
    Iterator iter;
    int i;
    
    for (i = 0, iter = nums.iterator(); iter.hasNext(); i++) {
      Number num = (Number) iter.next();
      result[i] = num.intValue();
    }
    
    return result;
  }
}
