/*
 * Copyright 2003 Federal Chancellery Austria
 * MOA-ID has been developed in a cooperation between BRZ, the Federal
 * Chancellery Austria - ICT staff unit, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * http://www.osor.eu/eupl/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 */


package at.gv.egovernment.moaspss.util;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.xml.sax.EntityResolver;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

/**
 * Implementation of the <code>org.xml.sax.EntityResolver</code>,
 * for use by a <code>org.apache.xerces.parsers.DOMParser</code>.
 * 
 * @author Patrick Peck
 * @version $Id$
 */
public class EntityResolverChain implements EntityResolver {
  /** The <code>EntityResolver</code>s in the chain. */
  private List resolvers = new ArrayList();

  /**
   * @see org.xml.sax.EntityResolver#resolveEntity(java.lang.String, java.lang.String)
   */
  public InputSource resolveEntity(String publicId, String systemId)
    throws SAXException, IOException {
    
    Iterator iter;
    
    for (iter = resolvers.iterator(); iter.hasNext(); ) {
      EntityResolver resolver = (EntityResolver) iter.next();
      InputSource is = resolver.resolveEntity(publicId, systemId);
      
      if (is != null) {
        return is;
      }
    }
    
    return null;
  }
  
  /**
   * Add an <code>EntityResolver</code> to the chain.
   * 
   * @param entityResolver The <code>EntityResolver</code> to add.
   */
  public void addEntityResolver(EntityResolver entityResolver) {
    resolvers.add(entityResolver);
  }

}
