/*
 * Copyright 2003 Federal Chancellery Austria
 * MOA-ID has been developed in a cooperation between BRZ, the Federal
 * Chancellery Austria - ICT staff unit, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * http://www.osor.eu/eupl/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 */


package at.gv.egovernment.moaspss.util;

import java.io.IOException;

import org.xml.sax.EntityResolver;
import org.xml.sax.ErrorHandler;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.helpers.DefaultHandler;

/**
 * A <code>DefaultHandler</code> that uses a <code>MOAEntityResolver</code> and
 * a <code>MOAErrorHandler</code>.
 * 
 * @author Patrick Peck
 * @version $Id$
 */
public class MOADefaultHandler extends DefaultHandler {
  /** The <code>EntityResolver</code> to use. */
  private EntityResolver entityResolver;
  /** The <code>ErrorHandler</code> to use. */
  private ErrorHandler errorHandler;

  /**
   * Create a new <code>MOADefaultHandler</code>.
   */
  public MOADefaultHandler() {
    entityResolver = new MOAEntityResolver();
    errorHandler = new MOAErrorHandler();
  }

  /**
   * Create a new <code>MOADefaultHandler</code>.
   * 
   * @param entityResolver The <code>EntityResolver</code> to use for resolving
   * external entities.
   * @param errorHandler The <code>ErrorHandler</code> to use for reporting
   * parsing errors.
   */
  public MOADefaultHandler(
    EntityResolver entityResolver,
    ErrorHandler errorHandler) {

    this.entityResolver = entityResolver;
    this.errorHandler = errorHandler;
  }

  /**
   * @see org.xml.sax.EntityResolver#resolveEntity(java.lang.String, java.lang.String)
   */
  public InputSource resolveEntity(String publicId, String systemId)
    throws SAXException {
    try {
      return entityResolver.resolveEntity(publicId, systemId);
    } catch (IOException e) {
      return null;
    }
  }

  /**
   * @see org.xml.sax.ErrorHandler#warning(org.xml.sax.SAXParseException)
   */
  public void warning(SAXParseException exception) throws SAXException {
    errorHandler.warning(exception);
  }

  /**
   * @see org.xml.sax.ErrorHandler#error(org.xml.sax.SAXParseException)
   */
  public void error(SAXParseException exception) throws SAXException {
    errorHandler.error(exception);
  }

  /**
   * @see org.xml.sax.ErrorHandler#fatalError(org.xml.sax.SAXParseException)
   */
  public void fatalError(SAXParseException exception) throws SAXException {
    errorHandler.fatalError(exception);
  }

}
