/*
 * Copyright 2003 Federal Chancellery Austria
 * MOA-ID has been developed in a cooperation between BRZ, the Federal
 * Chancellery Austria - ICT staff unit, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * http://www.osor.eu/eupl/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 */


package at.gv.egovernment.moaspss.util;

import java.util.List;

import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * A <code>NodeList</code> implementation based on a <code>List</code>.
 * 
 * @see java.util.List
 * @see org.w3c.dom.NodeList
 * 
 * @author Patrick Peck
 * @version $Id$
 */
public class NodeListAdapter implements NodeList {
  /** The <code>List</code> to wrap. */
  private List nodeList;
  
  /**
   * Create a new <code>NodeListAdapter</code>.
   * 
   * @param nodeList The <code>List</code> containing the nodes. 
   */
  public NodeListAdapter(List nodeList) {
    this.nodeList = nodeList;
  }

  /**
   * @see org.w3c.dom.NodeList#item(int)
   */
  public Node item(int index) {
    return (Node) nodeList.get(index);
  }

  /**
   * @see org.w3c.dom.NodeList#getLength()
   */
  public int getLength() {
    return nodeList.size();
  }

}
