/*
 * Copyright 2003 Federal Chancellery Austria
 * MOA-ID has been developed in a cooperation between BRZ, the Federal
 * Chancellery Austria - ICT staff unit, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * http://www.osor.eu/eupl/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 */


/*
 * Created on 26.04.2004
 *
 * @author rschamberger
 * $ID$
 */
package at.gv.egovernment.moaspss.util;

import org.w3c.dom.Element;

import at.gv.egovernment.moaspss.logging.Logger;

/**
 * utility functions to write XML data to files
 * @author rschamberger
 * @version $Id$
 */
public class OutputXML2File {

	/**
	 * writes an XML structure to file if debug is enabled in hierarchy (Encoding: UTF-8)
	 * 
	 * @param filename file name
	 * @param rootElem root element in DOM tree
	 * @param hierarchy of the Logger
	 */
	public static void debugOutputXML2File(String filename, Element rootElem, String hierarchy) {
		if (Logger.isDebugEnabled(hierarchy)) {
			outputXML2File(filename, rootElem);
		}
	}
	
	/**
	 * writes an XML structure to file if debug is enabled in hierarchy (Encoding: UTF-8)
	 * 
	 * @param filename file name
	 * @param xmlString XML string
	 * @param hierarchy of the Logger 
	 */
	public static void debugOutputXML2File(String filename, String xmlString, String hierarchy) {
		if (Logger.isDebugEnabled(hierarchy)) {
			outputXML2File(filename, xmlString);
		}
	}

	/**
	 * writes an XML structure to file (Encoding: UTF-8)
	 * 
	 * @param filename file name
	 * @param rootElem root element in DOM tree
	 */
	public static void outputXML2File(String filename, Element rootElem) {
		try {
			String xmlString = new String(DOMUtils.serializeNode(rootElem));
			outputXML2File(filename, xmlString);
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}
	
	/**
	 * writes an XML structure to file (Encoding: UTF-8)
	 * 
	 * @param filename file name
	 * @param xmlString XML string
	 */
	public static void outputXML2File(String filename, String xmlString) {
		try {
			java.io.OutputStream fout = new java.io.FileOutputStream(filename);
			byte[] xmlData = xmlString.getBytes("UTF-8");
			fout.write(xmlData);
			fout.close();
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}

}
