/*
 * Copyright 2003 Federal Chancellery Austria
 * MOA-ID has been developed in a cooperation between BRZ, the Federal
 * Chancellery Austria - ICT staff unit, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * http://www.osor.eu/eupl/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 */


package test.at.gv.egovernment.moa;

import java.io.FileInputStream;
import java.io.StringReader;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Document;

import org.xml.sax.InputSource;

import at.gv.egovernment.moaspss.util.Constants;
import at.gv.egovernment.moaspss.util.DOMUtils;
import junit.framework.TestCase;

/**
 * Base class for MOA test cases.
 * 
 * Provides some utility functions.
 * 
 * @author Patrick Peck
 * @version $Id$
 */
public class MOATestCase extends TestCase {

  protected static final String TESTDATA_ROOT = "data/test/";

  /**
   * Constructor for MOATestCase.
   * @param arg0
   */
  public MOATestCase(String name) {
    super(name);
  }

  /**
   * Parse an XML file non-validating.
   */
  public static Document parseXml(String fileName) throws Exception {
    return DOMUtils.parseDocument(
      new FileInputStream(fileName),
      false,
      null,
      null);
  }

  /**
   * Parse an XML validating with a given file name.
   * 
   * Uses the local schema resources.
   */
  public static Document parseXmlValidating(String fileName) throws Exception {
    return DOMUtils.parseDocument(
      new FileInputStream(fileName),
      true,
      Constants.ALL_SCHEMA_LOCATIONS,
      null);
  }

  /**
   * Parse an XML from a String.
   */
  public static Document parseXmlString(String xml) throws Exception {
    DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
    DocumentBuilder builder;
    
    factory.setNamespaceAware(true);
    builder = factory.newDocumentBuilder();

    return builder.parse(new InputSource(new StringReader(xml)));
  }

}
