/*
 * Copyright 2003 Federal Chancellery Austria
 * MOA-SPSS has been developed in a cooperation between BRZ, the Federal
 * Chancellery Austria - ICT staff unit, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * http://www.osor.eu/eupl/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 */


package at.gv.egovernment.moa.spss;
/**
 * Base class of application specific MOA exceptions.
 * 
 * Application exceptions are exceptions that originate from application
 * code (e.g. inconsistent data provided by the user, no permission to access
 * certain resources, etc.)
 * 
 * @author Patrick Peck
 * @version $Id$
 */
public class MOAApplicationException extends MOAException {
  
  /**
	 * 
	 */
	private static final long serialVersionUID = 1042877795934327684L;

/**
   * Create a new <code>MOAApplicationException</code>.
   * 
   * @param messageId The identifier of the message associated with this 
   * exception.
   * @param parameters Additional message parameters.
   */
  public MOAApplicationException(String messageId, Object[] parameters) {
    super(messageId, parameters);
  }

  /**
   * Create a new <code>MOAApplicationException</code>.
   * 
   * @param messageId The identifier of the message associated with this 
   * <code>MOAApplicationException</code>.
   * @param parameters Additional message parameters.
   * @param wrapped The exception wrapped by this
   * <code>MOAApplicationException</code>.
   */
  public MOAApplicationException(
    String messageId,
    Object[] parameters,
    Throwable wrapped) {
    super(messageId, parameters, wrapped);
  }

}
