/*
 * Copyright 2003 Federal Chancellery Austria
 * MOA-SPSS has been developed in a cooperation between BRZ, the Federal
 * Chancellery Austria - ICT staff unit, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * http://www.osor.eu/eupl/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 */


package at.gv.egovernment.moa.spss.api;

import at.gv.egovernment.moa.spss.MOAException;
import at.gv.egovernment.moa.spss.server.init.ConfiguratorImpl;

/**
 * Configures the SP/SS API.
 * 
 * Also handles dynamic configuration updates. 
 * 
 * @author Patrick Peck
 * @version $Id$
 */
public abstract class Configurator {

  /** The default implementation class. */
  private static final String DEFAULT_IMPLEMENTATION =
    "at.gv.egovernment.moa.spss.server.init.ConfiguratorImpl";

  /** The single instance of this class. */
  private static Configurator instance = null;

  public static synchronized Configurator getInstance() {
    if (instance == null) {
      try {
        /*DiscoverClass discover = new DiscoverClass();
        instance =
          (Configurator) discover.newInstance(
            Configurator.class,
            DEFAULT_IMPLEMENTATION);*/
    	instance = new ConfiguratorImpl();
      } catch (Exception e) {
        // this can not happen since we provide a valid default 
        // implementation
      }
    }
    return instance;
  }

  /**
   * Initialize the SP/SS configuration.
   * 
   * Only a single call to this method will have an effect. Use 
   * <code>update()</code> for reflecting changes in the configuration instead. 
   * 
   * @throws MOAException An error occurred updating the SP/SS configuration.
   */  
  public abstract void init() throws MOAException;
  
  /**
   * Update the SP/SS configuration.
   * 
   * This will only have an effect after the system has been initialized once
   * using <code>init()</code>.
   * 
   * @throws MOAException An error occurred updating the SP/SS configuration.
   */
  public abstract void update() throws MOAException;

}
