/*
 * Copyright 2003 Federal Chancellery Austria
 * MOA-SPSS has been developed in a cooperation between BRZ, the Federal
 * Chancellery Austria - ICT staff unit, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * http://www.osor.eu/eupl/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 */


package at.gv.egovernment.moa.spss.api;

import at.gv.egovernment.moa.spss.MOAException;

import at.gv.egovernment.moa.spss.api.xmlsign.CreateXMLSignatureRequest;
import at.gv.egovernment.moa.spss.api.xmlsign.CreateXMLSignatureResponse;
import at.gv.egovernment.moa.spss.server.invoke.SignatureCreationServiceImpl;

/**
 * Interface providing functions for signature creation.
 * 
 * @author Patrick Peck
 * @version $Id$
 */
public abstract class SignatureCreationService {

  /** The default implementation class. */
  private static final String DEFAULT_IMPLEMENTATION =
    "at.gv.egovernment.moa.spss.server.invoke.SignatureCreationServiceImpl";
  
  /** The single instance of this class. */  
  private static SignatureCreationService instance = null;

  /**
   * Get an instance of the <code>SignatureCreationService</code>.
   * 
   * @return A concrete instance of the <code>SignatureCreationService</code>.
   */
  public static synchronized SignatureCreationService getInstance() {
    if (instance == null) {
      try {
    	  /*
        DiscoverClass discover = new DiscoverClass();
        instance =
          (SignatureCreationService) discover.newInstance(
            SignatureCreationService.class,
            DEFAULT_IMPLEMENTATION);*/
        instance = new SignatureCreationServiceImpl();
      } catch (Exception e) {
        // this can not happen since we provide a valid default 
        // implementation
      }
    }
    return instance;
  }
  
  /**
   * Create an XML signature.
   * 
   * @param request Information on how to create the signature.
   * @return A <code>CreateXMLSignatureResponse</code> containing the 
   * signature.
   * @throws MOAException Error in server side MOA module.
   */
  public abstract CreateXMLSignatureResponse createXMLSignature(CreateXMLSignatureRequest request)
    throws MOAException;
}
