/*
 * Copyright 2003 Federal Chancellery Austria
 * MOA-SPSS has been developed in a cooperation between BRZ, the Federal
 * Chancellery Austria - ICT staff unit, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * http://www.osor.eu/eupl/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 */


package at.gv.egovernment.moa.spss.api;

import at.gv.egovernment.moa.spss.MOAException;
import at.gv.egovernment.moa.spss.api.cmsverify.VerifyCMSSignatureRequest;
import at.gv.egovernment.moa.spss.api.cmsverify.VerifyCMSSignatureResponse;
import at.gv.egovernment.moa.spss.api.xmlverify.VerifyXMLSignatureRequest;
import at.gv.egovernment.moa.spss.api.xmlverify.VerifyXMLSignatureResponse;
import at.gv.egovernment.moa.spss.server.invoke.SignatureVerificationServiceImpl;

/**
 * Interface providing functions for verifying signatures.
 * 
 * @author Patrick Peck
 * @version $Id$
 */
public abstract class SignatureVerificationService {

  /** The default implementation class. */
  private static final String DEFAULT_IMPLEMENTATION =
    "at.gv.egovernment.moa.spss.server.invoke.SignatureVerificationServiceImpl";

  /** The single instance of this class. */
  private static SignatureVerificationService instance = null;

  /**
   * Get an instance of the <code>SignatureVerificationService</code>.
   * 
   * @return A concrete instance of the 
   * <code>SignatureVerificationService</code>.
   */
  public static synchronized SignatureVerificationService getInstance() {
    if (instance == null) {
      try {
        /*DiscoverClass discover = new DiscoverClass();
        instance =
          (SignatureVerificationService) discover.newInstance(
            SignatureVerificationService.class,
            DEFAULT_IMPLEMENTATION);*/
    	  instance = new SignatureVerificationServiceImpl();
      } catch (Exception e) {
        // this can not happen since we provide a valid default 
        // implementation
      }
    }
    return instance;
  }

  /**
   * Verify a CMS signature.
   * 
   * @param request Detailed information on the verification that should be
   * performed.
   * @return A <code>VerifyCMSSignatureResponse</code> object that contains
   * information about the performed verification.
   * @throws MOAException Error in server side MOA module.
   */
  public abstract VerifyCMSSignatureResponse verifyCMSSignature(VerifyCMSSignatureRequest request)
    throws MOAException;
  /**
   * Verfiy an XML Signature.
   * 
   * @param request  information on the verification that should be performed.
   * @return A <code>VerifyXMLSignatureResponse</code> object that contains
   * information about the performed verification.
   * @throws MOAException Error in server side MOA module.
   */
  public abstract VerifyXMLSignatureResponse verifyXMLSignature(VerifyXMLSignatureRequest request)
    throws MOAException;
}
