/*
 * Copyright 2003 Federal Chancellery Austria
 * MOA-SPSS has been developed in a cooperation between BRZ, the Federal
 * Chancellery Austria - ICT staff unit, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * http://www.osor.eu/eupl/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 */


package at.gv.egovernment.moa.spss.api.cmsverify;

import java.io.InputStream;
import java.util.Date;

/**
 * Object that encapsulates a request to verify a CMS signature.
 * 
 * @author Patrick Peck
 * @author Stephan Grill
 * @version $Id$
 */
public interface VerifyCMSSignatureRequest {
  /**
   * Indicates, that signature checks for all signatories must be returned.
   */
  public static int[] ALL_SIGNATORIES = new int[] { -1 };
  /**
   * Gets the positions of signatories whose signature must be verified.
   * 
   * @return The positions of signatories.
   */
  public int[] getSignatories();
  /**
   * Gets the date and time for which the signature verification has to 
   * be performed.
   * 
   * @return Date and time for which the signature verification has
   * to be performed.
   */
  public Date getDateTime();
  /**
   * Gets the binary CMS signature.
   * 
   * @return An <code>InputStream</code> from which the binary CMS signature
   * can be read.
   */
  public InputStream getCMSSignature();
  /** 
   * Gets the data object necessary for the verification.
   * 
   * @return The data object necessary for verification.
   */
  public CMSDataObject getDataObject();
  /**
   * Gets the profile ID of trusted certificates to be used for signature
   * verification.
   * 
   * @return The profile ID of trusted certificates.
   */
  public String getTrustProfileId();
  
  public boolean isPDF();
  public boolean isExtended();
}
