/*
 * Copyright 2003 Federal Chancellery Austria
 * MOA-SPSS has been developed in a cooperation between BRZ, the Federal
 * Chancellery Austria - ICT staff unit, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * http://www.osor.eu/eupl/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 */


package at.gv.egovernment.moa.spss.api.common;

/**
 * Encapsulates content data.
 * 
 * @author Patrick Peck
 * @author Stephan Grill
 * @author Gregor Karlinger
 * @version $Id$
 */
public interface Content {

  /**
   * Indicates that this object contains a URI reference to some content. 
   */
  public static final int REFERENCE_CONTENT = 0;

  /**
   * Indicates that this object contains binary content.
   */
  public static final int BINARY_CONTENT = 1;

  /**
   * Indicates that this object contains XML content.
   */
  public static final int XML_CONTENT = 2;
 
  /**
   * Indicates that this object contains a location reference content.
   */
  public static final int LOCREF_CONTENT = 3;

  /**
   * Gets the type of content contained in this object.
   * 
   * @return The type of content, one of <code>BINARY_CONTENT</code>, <code>XML_CONTENT</code>, <code>
   *         REFERENCE_CONTENT</code> or <code>LOCREF_CONTENT</code>.
   */
  public int getContentType();

  /**
   * Gets the reference to the content data (a URI).
   * 
   * @return The reference to the content data.
   */
  public String getReference();
}
