/*
 * Copyright 2003 Federal Chancellery Austria
 * MOA-SPSS has been developed in a cooperation between BRZ, the Federal
 * Chancellery Austria - ICT staff unit, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * http://www.osor.eu/eupl/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 */


package at.gv.egovernment.moa.spss.api.common;

/**
 * Interface specifying accessors for two attributes needed for returning 
 * <code>HashInputData</code> and <code>ReferenceInputData</code> information
 * as part of <code>VerifyXMLSignatureResponse</code>.
 * 
 * @author Gregor Karlinger
 * 
 * @version $Id$
 */
public interface InputData extends Content
{
  /**
   * Possible value returned by {@link #getPartOf}. 
   */
  public static String CONTAINER_SIGNEDINFO_ = "SignedInfo";

  /**
   * Possible value returned by {@link #getPartOf}. 
   */
  public static String CONTAINER_XMLDSIGMANIFEST_ = "XMLDSIGManifest";
  
  /**
   * Value returned by {link getReferringReferenceNumber}, signalling that the
   * attribute is not used.
   */
  public static int REFERER_NONE_ = -1;
  
  /**
   * Returns a <code>String</code> signalling what kind of container the 
   * XMLDSIG <code>Reference</code> this <code>InputData</code> belongs 
   * to is part of.
   * 
   * @return the kind of container. 
   */
  public String getPartOf();
  
  /**
   * If this <code>InputData</code> belongs to an XMLDSIG <code>Reference</code>
   * being part of either a XMLDSIGManifest or a SignatureManifest, this method
   * returns a positive int value signalling the particular <code>Reference</code>
   * of the XMLDSIG <code>SignedInfo</code> referring to the XMLDSIGManifest or
   * SignatureManifest respectively.
   */
  public int getReferringReferenceNumber();
}
