/*
 * Copyright 2003 Federal Chancellery Austria
 * MOA-SPSS has been developed in a cooperation between BRZ, the Federal
 * Chancellery Austria - ICT staff unit, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * http://www.osor.eu/eupl/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 */


package at.gv.egovernment.moa.spss.api.common;

import java.security.cert.X509Certificate;


/**
 * Contains information about the signer.
 * 
 * @author Patrick Peck
 * @author Stephan Grill
 * @version $Id$
 */
public interface SignerInfo {
  /**
   * Gets the signer certificate.
   * 
   * @return The signer certificate.
   */  
  public X509Certificate getSignerCertificate();
  /**
   * Checks, whether the certificate contained in this object is qualified.
   * 
   * @return <code>true</code>, if the certificate is qualified, otherwise
   * <code>false</code>.
   */
  public boolean isQualifiedCertificate();
  
  
  /**
   * Checks, whether the signature is based on a SSCD.
   * 
   * @return <code>true</code>, if the signature is based on a SSCD, otherwise
   * <code>false</code>.
   */
  public boolean isSSCD();
  
  /**
   * Returns the source of the SSCD check (TSL or Certificate)   * 
   */
  public String getSSCDSource();

  /**
   * Returns the source of the QC check (TSL or Certificate)   * 
   */
  public String getQCSource();

  /**
   * Returns the signer certificate issuer country code
   * @return
   */
  public String getIssuerCountryCode();
  /**
   * Checks, whether the certificate contained in this object is a 
   * public authority certificate.
   * 
   * @return <code>true</code>, if the certificate is a public authority 
   * certificate, otherwise <code>false</code>.
   */
  public boolean isPublicAuthority();
  /**
   * Gets the public authority ID, if the certificate contained in this
   * object is from a public authority.
   * 
   * @return The public authority ID.
   */  
  public String getPublicAuhtorityID();

}
