/*
 * Copyright 2003 Federal Chancellery Austria
 * MOA-SPSS has been developed in a cooperation between BRZ, the Federal
 * Chancellery Austria - ICT staff unit, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * http://www.osor.eu/eupl/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 */


package at.gv.egovernment.moa.spss.api.common;


import iaik.xml.crypto.utils.URI;

import java.util.Date;


/**
 * Contains TSL configuration information.
 * 
 * @author kstranacher
 */
public interface TSLConfiguration {
	
	/** Default URL of EU TSL */
	public String DEFAULT_EU_TSL_URL = "https://ec.europa.eu/information_society/policy/esignature/trusted-list/tl-mp.xml";
	
	/** Default period (1day=86400000 msec) for update schedule */
	public String DEFAULT_UPDATE_SCHEDULE_PERIOD = "86400000";
	
	/** Default start time (2:00 AM) for update schedule */
	public String DEFAULT_UPDATE_SCHEDULE_STARTTIME = "02:00:00";
	
	public String DEFAULT_WORKING_DIR = "tslworking";
	
  /**
   * Gets the EU TSL URL.
   * 
   * @return The EU TSL URL.
   */  
  public String getEuTSLUrl();

  /**
   * 
   * @return
   */
  public Date getUpdateScheduleStartTime();
  
  /**
   * 
   * @return
   */
  public long getUpdateSchedulePeriod();
  
  /**
   * 
   * @return
   */
  public String getWorkingDirectory();
  
  /**
   * 
   * @return
   */
  public URI getWorkingDirectoryAsURI();

}
