/*
 * Copyright 2003 Federal Chancellery Austria
 * MOA-SPSS has been developed in a cooperation between BRZ, the Federal
 * Chancellery Austria - ICT staff unit, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * http://www.osor.eu/eupl/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 */


package at.gv.egovernment.moa.spss.api.common;

import java.util.Map;

/**
 * An XPath expression set operation.
 * 
 * @author Patrick Peck
 * @version $Id$
 */
public interface XPathFilter {
  /** Subtract this filter's node set from the resulting node set. */
  public static final String SUBTRACT_TYPE = "subtract";
  /** Intersect this filter's node set with the resulting node set. */
  public static final String INTERSECT_TYPE = "intersect";
  /** Compute the union of this filter's node set and the resulting node set. */
  public static final String UNION_TYPE = "union";
  
  /**
   * Gets the type of this <code>XPathFilter</code>.
   * 
   * @return The type of this <code>XPathFilter</code>.
   */
  public String getFilterType();
  /**
   * Gets the XPath expression for selecting the nodes.
   * 
   * @return The XPath expression for selecting the nodes.
   */
  public String getXPathExpression();
  /**
   * Gets The namespace prefix to URI mapping used during evaluation of the
   * XPath expression.
   * 
   * @return The namespace prefix to URI mapping.
   */
  public Map getNamespaceDeclarations();
}
