/*
 * Copyright 2003 Federal Chancellery Austria
 * MOA-SPSS has been developed in a cooperation between BRZ, the Federal
 * Chancellery Austria - ICT staff unit, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * http://www.osor.eu/eupl/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 */


package at.gv.egovernment.moa.spss.api.common;

import java.util.Map;

/**
 * A <code>Transform</code> performing an XPath selection.
 * 
 * @author Patrick Peck
 * @version $Id$
 */
public interface XPathTransform extends Transform {
  /** Algorithm URI of the XPath <code>Transform</code>. */
  public static final String XPATH =
    "http://www.w3.org/TR/1999/REC-xpath-19991116";

  /**
   * Gets the XPath expression used for selection.
   * 
   * @return The XPath expression used for selection.
   */
  public String getXPathExpression();
  /**
   * Gets The namespace prefix to URI mapping used during evaluation of the
   * XPath expression.
   * 
   * @return The namespace prefix to URI mapping.
   */
  public Map getNamespaceDeclarations();
  
}
