/*
 * Copyright 2003 Federal Chancellery Austria
 * MOA-SPSS has been developed in a cooperation between BRZ, the Federal
 * Chancellery Austria - ICT staff unit, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * http://www.osor.eu/eupl/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 */


package at.gv.egovernment.moa.spss.api.impl;

import java.math.BigDecimal;

import at.gv.egovernment.moa.spss.api.cmsverify.CMSContent;
import at.gv.egovernment.moa.spss.api.cmsverify.CMSDataObject;
import at.gv.egovernment.moa.spss.api.common.MetaInfo;

/**
 * Default implementation of <code>CMLSDataObject</code>.
 * 
 * @author Fatemeh Philippi
 * @version $Id$
 */
public class CMSDataObjectImpl implements CMSDataObject {
  
  /** The <code>MetaInfo</code> associated with the CMS data object. */
  private MetaInfo metaInfo;
  /** The <code>CMSContent</code> contained in this data object. */
  private CMSContent cmsContent;
  
  private BigDecimal excludeByteRangeFrom;
  private BigDecimal excludeByteRangeTo;

  /**
   * Sets the meta information associated with the CMS data object.
   * 
   * @param metaInfo The meta information.
   */
  public void setMetaInfo(MetaInfo metaInfo) {
    this.metaInfo = metaInfo;
  }

  public MetaInfo getMetaInfo() {
    return metaInfo;
  }

  /**
   * Sets the data of this <code>CMSDataObject</code>.
   * 
   * @param cmsContent The actual data of this <code>CMSDataObject</code>.
   */
  public void setContent(CMSContent cmsContent) {
    this.cmsContent = cmsContent;
  }

  public CMSContent getContent() {
    return cmsContent;
  }

  public void setExcludeByteRangeFrom(BigDecimal excludeByteRangeFrom) {
  	this.excludeByteRangeFrom = excludeByteRangeFrom;
  }
  
  public BigDecimal getExcludeByteRangeFrom() {
	  return excludeByteRangeFrom;
  }
  
  public void setExcludeByteRangeTo(BigDecimal excludeByteRangeTo) {
	  this.excludeByteRangeTo = excludeByteRangeTo;
  }
  
  public BigDecimal getExcludeByteRangeTo() {
	  return excludeByteRangeTo;
  }	
  
}
