/*
 * Copyright 2003 Federal Chancellery Austria
 * MOA-SPSS has been developed in a cooperation between BRZ, the Federal
 * Chancellery Austria - ICT staff unit, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * http://www.osor.eu/eupl/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 */


package at.gv.egovernment.moa.spss.api.impl;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import at.gv.egovernment.moa.spss.api.xmlsign.CreateXMLSignatureRequest;

/**
 * Default implementation of <code>CreateXMLSignatureRequest</code>.
 * 
 * @author Fatemeh Philippi
 * @version $Id$
 */
public class CreateXMLSignatureRequestImpl
  implements CreateXMLSignatureRequest {

  /** The identifier for selecting the private keys for creating the signature.*/
  private String keyIdentifier;
  /** Information for creating a single signature. */
  private List singleSignatureInfos = new ArrayList();

  /**
   * Sets the identifier for selecting the private keys for creating the 
   * signature.
   * 
   * @param keyIdentifier The identifier for selecting the private keys.
   */
  public void setKeyIdentifier(String keyIdentifier) {
    this.keyIdentifier = keyIdentifier;
  }

  public String getKeyIdentifier() {
    return keyIdentifier;
  }

  /**
   * Sets the information for creating single signatures.
   * 
   * @param singleSignaureInfos The information for creating single signatures.
   */
  public void setSingleSignatureInfos(List singleSignaureInfos) {
    this.singleSignatureInfos =
      singleSignaureInfos != null
        ? Collections.unmodifiableList(new ArrayList(singleSignaureInfos))
        : null;
  }

  public List getSingleSignatureInfos() {
    return singleSignatureInfos;
  }

}
