/*
 * Copyright 2003 Federal Chancellery Austria
 * MOA-SPSS has been developed in a cooperation between BRZ, the Federal
 * Chancellery Austria - ICT staff unit, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * http://www.osor.eu/eupl/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 */


package at.gv.egovernment.moa.spss.api.impl;

import at.gv.egovernment.moa.spss.api.common.Content;
import at.gv.egovernment.moa.spss.api.xmlsign.CreateTransformsInfoProfile;
import at.gv.egovernment.moa.spss.api.xmlsign.DataObjectInfo;

/**
 * Default implementation of <code>DataObjectInfo</code>.
 * 
 * @author Fatemeh Philippi
 * @version $Id$
 */
public class DataObjectInfoImpl implements DataObjectInfo {
  /** The signature structure type. */
  private String stucture;
  /** Whether a reference will be placed in the signature itself or in the
   * manifest */
  private boolean childOfManifest;
  /** The data object to be signed. */
  private Content dataObject;
  /** The profile containing additional information for the transformations. */
  private CreateTransformsInfoProfile createTransformsInfoProfile;

  /**
   * Sets the signature structure type.
   * 
   * @param structure The signature structure type.
   */
  public void setStructure(String structure) {
    this.stucture = structure;
  }

  public String getStructure() {
    return stucture;
  }

  /**
   * Sets whether a reference will be placed in the signature itself or in the
   * manifest.
   * 
   * @param childOfManifest Whether to put the reference in the signature of
   * in the manifest. 
   */
  public void setChildOfManifest(boolean childOfManifest) {
    this.childOfManifest = childOfManifest;
  }

  public boolean isChildOfManifest() {
    return childOfManifest;
  }

  /**
   * Sets the data object to be signed.
   * 
   * @param dataObject The data object to be signed.
   */
  public void setDataObject(Content dataObject) {
    this.dataObject = dataObject;
  }

  public Content getDataObject() {
    return dataObject;
  }

  /**
   * Sets additional information for the transformations.
   * 
   * @param profile The profile containing additional information for the
   * transformations.
   */
  public void setCreateTransformsInfoProfile(CreateTransformsInfoProfile profile) {
    this.createTransformsInfoProfile = profile;
  }

  public CreateTransformsInfoProfile getCreateTransformsInfoProfile() {
    return createTransformsInfoProfile;
  }

}
