/*
 * Copyright 2003 Federal Chancellery Austria
 * MOA-SPSS has been developed in a cooperation between BRZ, the Federal
 * Chancellery Austria - ICT staff unit, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * http://www.osor.eu/eupl/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 */


package at.gv.egovernment.moa.spss.api.impl;

import org.w3c.dom.NodeList;

import at.gv.egovernment.moa.spss.MOARuntimeException;
import at.gv.egovernment.moa.spss.api.common.Content;
import at.gv.egovernment.moa.spss.api.common.ContentXML;
import at.gv.egovernment.moa.spss.api.common.InputData;

/**
 * Content wrapper decorating an XML content with two additional attributes 
 * needed for returning <code>HashInputData</code> and <code>ReferenceInputData
 * </code> information as part of <code>VerifyXMLSignatureResponse</code>.
 * 
 * @author Gregor Karlinger
 * 
 * @version $Id$
 */
public class InputDataXMLImpl implements ContentXML, InputData
{
  /**
   * The wrapped <code>ContentXML</code>.
   */
  protected ContentXML wrapped_;
  
  /**
   * This attribute signals what kind of container the XMLDSIG <code>Reference</code>
   * this <code>InputData</code> belongs to is part of. 
   */
  protected String partOf_;
  
  /**
   * If this <code>InputData</code> belongs to an XMLDSIG <code>Reference</code>
   * being part of either a XMLDSIGManifest or a SignatureManifest, this attribute
   * (a positive int) signals the particular <code>Reference</code> of the XMLDSIG 
   * <code>SignedInfo</code> referring to the XMLDSIGManifest or SignatureManifest
   * respectively.
   */
  protected int referringReferenceNumber_;
  
  /**
   * Creates a new instance.
   * 
   * @param wrapped The wrapped <code>ContentBinary</code>. Must be of type {@link Content#XML_CONTENT}.
   * 
   * @param partOf see {@link InputData}
   * 
   * @param referringReferenceNumber see {@link InputData}
   */
  public InputDataXMLImpl(Content wrapped, String partOf, int referringReferenceNumber)
  {
    if (wrapped.getContentType() != Content.XML_CONTENT) throw new MOARuntimeException("9901", null);

    wrapped_ = (ContentXML) wrapped;
    partOf_ =  partOf;
    referringReferenceNumber_ = referringReferenceNumber;
  }
  
  /** 
   * @see at.gv.egovernment.moa.spss.api.common.Content#getContentType()
   */
  public int getContentType()
  {
    return wrapped_.getContentType();
  }

  /**
   * @see at.gv.egovernment.moa.spss.api.common.Content#getReference()
   */
  public String getReference()
  {
    return wrapped_.getReference();
  }

  /**
   * @see at.gv.egovernment.moa.spss.api.common.ContentXML#getXMLContent()
   */
  public NodeList getXMLContent()
  {
    return wrapped_.getXMLContent();
  }

  /**
   * @see at.gv.egovernment.moa.spss.api.common.InputData#getPartOf()
   */
  public String getPartOf()
  {
    return partOf_;
  }

  /**
   * @see at.gv.egovernment.moa.spss.api.common.InputData#getReferringReferenceNumber()
   */
  public int getReferringReferenceNumber()
  {
    return referringReferenceNumber_;
  }

}
