/*
 * Copyright 2003 Federal Chancellery Austria
 * MOA-SPSS has been developed in a cooperation between BRZ, the Federal
 * Chancellery Austria - ICT staff unit, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * http://www.osor.eu/eupl/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 */


package at.gv.egovernment.moa.spss.api.impl;

import org.w3c.dom.NodeList;

import at.gv.egovernment.moa.spss.api.common.MetaInfo;

/**
 * Default implementation of <code>MetaInfo</code>.
 * @author Fatemeh Philippi
 * @version $Id$
 */
public class MetaInfoImpl implements MetaInfo {
  /** Information about the MIME type. */
  private String mimeType;
  /** URI pointing to a description of the content. */
  private String description;
  /** Descriptive XML content. */
  private NodeList anyElements;
  /** Type information for XML signature creation */
  private String type;

  /**
   * Sets the MIME type.
   * 
   * @param mimeType The MIME type to set.
   */
  public void setMimeType(String mimeType) {
    this.mimeType = mimeType;
  }

  public String getMimeType() {
    return mimeType;
  }

  /**
   * Sets the URI pointing to a description of the content.
   * 
   * @param description The URI pointing to a description of the content.
   */
  public void setDescription(String description) {
    this.description = description;
  }

  public String getDescription() {
    return description;
  }

  /**
   * Sets descriptive XML content.
   * 
   * @param anyElements The elements to set.
   */
  public void setAnyElements(NodeList anyElements) {
    this.anyElements = anyElements; 
  }

  public NodeList getAnyElements() {
    return anyElements;
  }

  /**
   * Sets the XML signature creation type information.
   * 
   * @param type the XML signature creation type information to set.
   */
  public void setType(String type) {
    this.type = type;
  }

  public String getType() {
    return type;
  }


}
