/*
 * Copyright 2003 Federal Chancellery Austria
 * MOA-SPSS has been developed in a cooperation between BRZ, the Federal
 * Chancellery Austria - ICT staff unit, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * http://www.osor.eu/eupl/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 */


package at.gv.egovernment.moa.spss.api.impl;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import at.gv.egovernment.moa.spss.api.xmlverify.SignatureManifestCheckParams;

/**
 * Default implementation of <code>SignatureManifestCheckParams</code>.
 * 
 * @author Fatemeh Philippi
 * @version $Id$
 */
public class SignatureManifestCheckParamsImpl
  implements SignatureManifestCheckParams {

  /** Referential information. */
  private List referenceInfos;
  /** Whether to return the signature source data. */
  private boolean returnReferenceInputData = true;

  /**
   * Sets the referantial information.
   * 
   * @param referenceInfos The referential information.
   */
  public void setReferenceInfos(List referenceInfos) {
    this.referenceInfos =
      referenceInfos != null
        ? Collections.unmodifiableList(new ArrayList(referenceInfos))
        : null;
  }

  public List getReferenceInfos() {
    return referenceInfos;
  }
  
  /**
   * Sets whether to return signature source data.
   * 
   * @param returnReferenceInputData Whether to return signature source data.
   */
  public void setReturnReferenceInputData(boolean returnReferenceInputData) {
    this.returnReferenceInputData = returnReferenceInputData;
  }

  public boolean getReturnReferenceInputData() {
    return returnReferenceInputData;
  }

}
