/*
 * Copyright 2003 Federal Chancellery Austria
 * MOA-SPSS has been developed in a cooperation between BRZ, the Federal
 * Chancellery Austria - ICT staff unit, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * http://www.osor.eu/eupl/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 */


package at.gv.egovernment.moa.spss.api.impl;

import java.util.List;

import at.gv.egovernment.moa.spss.api.cmsverify.VerifyCMSSignatureResponseElement;
import at.gv.egovernment.moa.spss.api.common.CheckResult;
import at.gv.egovernment.moa.spss.api.common.SignerInfo;

/**
 * Default implementation of <code>VerifyCMSSignatureResponseElement</code>.
 * 
 * @author Fatemeh Philippi
 * @version $Id$
 */
public class VerifyCMSSignatureResponseElementImpl
  implements VerifyCMSSignatureResponseElement {

  /** Information about the signer certificate. */
  private SignerInfo signerInfo;
  /** Information about the signature check. */
  private CheckResult signatureCheck;
  /** Information about the certificate check. */
  private CheckResult certificateCheck;
  
  private List adesResults = null;
  
  /**
   * Sets a SignerInfo element according to CMS.
   * 
   * @param signerInfo The SignerInfo element according to CMS.
   */
  public void setSignerInfo(SignerInfo signerInfo) {
    this.signerInfo = signerInfo;
  }

  public SignerInfo getSignerInfo() {
    return signerInfo;
  }

  /**
   * Sets a result of the signature verification.
   * 
   * @param signatureCheck The result of the signature verification.
   */
  public void setSignatureCheck(CheckResult signatureCheck) {
    this.signatureCheck = signatureCheck;
  }

  public CheckResult getSignatureCheck() {
    return signatureCheck;
  }

  /**
   * Sets a result of the certificate verification.
   * 
   * @param certificateCheck The result of the certificate verification.
   */
  public void setCertificateCheck(CheckResult certificateCheck) {
    this.certificateCheck = certificateCheck;
  }

  public CheckResult getCertificateCheck() {
    return certificateCheck;
  }

  public void setAdESFormResults(List adesResults) {
	this.adesResults = adesResults;
  }
  
  public List getAdESFormResults() {
	return adesResults;
  }
  
}
