/*
 * Copyright 2003 Federal Chancellery Austria
 * MOA-SPSS has been developed in a cooperation between BRZ, the Federal
 * Chancellery Austria - ICT staff unit, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * http://www.osor.eu/eupl/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 */


package at.gv.egovernment.moa.spss.api.impl;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import at.gv.egovernment.moa.spss.api.xmlverify.VerifyTransformsInfoProfileExplicit;

/**
 * Default implementation of <code>VerifyTransformsInfoProfileExplicit</code>.
 * 
 * @author Fatemeh Philippi
 * @version $Id$
 */
public class VerifyTransformsInfoProfileExplicitImpl
  implements VerifyTransformsInfoProfileExplicit {

  /** The transforms contained in this profile. */
  private List transforms;
  /** Additional information for the transforms. */
  private List transformParameters = new ArrayList();

  /**
   * Sets the transforms contained in this profile.
   * 
   * @param transforms The transforms contained in this profile.
   */
  public void setTransforms(List transforms) {
    this.transforms =
      transforms != null
        ? Collections.unmodifiableList(new ArrayList(transforms))
        : null;
  }

  public List getTransforms() {
    return transforms;
  }

  /**
   * Sets additional information for the transforms.
   * 
   * @param transformParameters Additional information for the transforms.
   */
  public void setTransformParameters(List transformParameters) {
    this.transformParameters = new ArrayList(transformParameters);
  }

  public List getTransformParameters() {
    return transformParameters;
  }

  /**
   * Gets the type of <code>VerifyTransformsInfoProfile</code>.
   * 
   * @return EXPLICIT_VERIFYTRANSFORMSINFOPROFILE
   */
  public int getVerifyTransformsInfoProfileType() {
    return EXPLICIT_VERIFYTRANSFORMSINFOPROFILE;
  }

}
