/*
 * Copyright 2003 Federal Chancellery Austria
 * MOA-SPSS has been developed in a cooperation between BRZ, the Federal
 * Chancellery Austria - ICT staff unit, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * http://www.osor.eu/eupl/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 */

package at.gv.egovernment.moa.spss.api.impl;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import at.gv.egovernment.moa.spss.api.common.CheckResult;
import at.gv.egovernment.moa.spss.api.common.SignerInfo;
import at.gv.egovernment.moa.spss.api.xmlverify.ReferencesCheckResult;
import at.gv.egovernment.moa.spss.api.xmlverify.VerifyXMLSignatureResponse;

/**
 * @author Fatemeh Philippi
 * @version $Id$
 */
public class VerifyXMLSignatureResponseImpl implements VerifyXMLSignatureResponse {

	/** Information about the signer certificate. */
	private SignerInfo signerInfo;

	/**
	 * The hash input data objects. The list consists of
	 * {@link at.gv.egovernment.moa.spss.api.common.InputData}s.
	 */
	private List hashInputDatas = new ArrayList();

	/**
	 * The reference input data objects. The list consists of
	 * {@link at.gv.egovernment.moa.spss.api.common.InputData}s.
	 */
	private List referenceInputDatas = new ArrayList();

	/**
	 * The list of form validation results
	 */
	private List adesFormResults = new ArrayList();

	/** Information about the signature check. */
	private ReferencesCheckResult signatureCheck;
	/** Information about the signature manifest check. */
	private ReferencesCheckResult signatureManifestCheck;
	/** Information about the XMLDsig manifest check. */
	private List xmlDsigManifestChecks = new ArrayList();
	/** Information about the certificate check. */
	private CheckResult certificateCheck;

	/**
	 * Sets information about the signer certificate.
	 * 
	 * @param signerInfo
	 *            Information about the signer certificate.
	 */
	public void setSignerInfo(SignerInfo signerInfo) {
		this.signerInfo = signerInfo;
	}

	public SignerInfo getSignerInfo() {
		return signerInfo;
	}

	/**
	 * Sets data signed by the signatory.
	 * 
	 * @param hashInputDatas
	 *            The signed datas.
	 */
	public void setHashInputDatas(List hashInputDatas) {
		this.hashInputDatas = hashInputDatas != null ? Collections.unmodifiableList(new ArrayList(hashInputDatas))
				: null;
	}

	public List getHashInputDatas() {
		return hashInputDatas;
	}

	/**
	 * Sets the source data elements.
	 * 
	 * @param referenceInputDatas
	 *            The source data elements.
	 */
	public void setReferenceInputDatas(List referenceInputDatas) {
		this.referenceInputDatas = referenceInputDatas != null
				? Collections.unmodifiableList(new ArrayList(referenceInputDatas)) : null;
	}

	public List getReferenceInputDatas() {
		return referenceInputDatas;
	}

	/**
	 * Sets the result of the signature verification.
	 * 
	 * @param signatureCheck
	 *            The result of the signature verification.
	 */
	public void setSignatureCheck(ReferencesCheckResult signatureCheck) {
		this.signatureCheck = signatureCheck;
	}

	public ReferencesCheckResult getSignatureCheck() {
		return signatureCheck;
	}

	/**
	 * Sets the result of the signature manifest verification.
	 * 
	 * @param signatureManifestCheck
	 *            The result of the signature manifest verification.
	 */
	public void setSignatureManifestCheck(ReferencesCheckResult signatureManifestCheck) {
		this.signatureManifestCheck = signatureManifestCheck;
	}

	public ReferencesCheckResult getSignatureManifestCheck() {
		return signatureManifestCheck;
	}

	/**
	 * Sets the result of the certification verification.
	 * 
	 * @param certificateCheck
	 *            The result of the certificate verification.
	 */
	public void setCertificateCheck(CheckResult certificateCheck) {
		this.certificateCheck = certificateCheck;
	}

	public CheckResult getCertificateCheck() {
		return certificateCheck;
	}

	/**
	 * Sets the XMLDSigManifestChecks.
	 * 
	 * @param xmlDsigManifestChecks
	 *            The XMLDSigManifestChecks.
	 */
	public void setXMLDsigManifestChecks(List xmlDsigManifestChecks) {
		this.xmlDsigManifestChecks = xmlDsigManifestChecks != null
				? Collections.unmodifiableList(new ArrayList(xmlDsigManifestChecks)) : null;
	}

	public List getXMLDsigManifestChecks() {
		return xmlDsigManifestChecks;
	}

	public void setAdESFormResults(List adesFormResults) {
		this.adesFormResults = adesFormResults;
	}
	
	@Override
	public List getAdESFormResults() {
		return this.adesFormResults;
	}

}
