/*
 * Copyright 2003 Federal Chancellery Austria
 * MOA-SPSS has been developed in a cooperation between BRZ, the Federal
 * Chancellery Austria - ICT staff unit, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * http://www.osor.eu/eupl/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 */


package at.gv.egovernment.moa.spss.api.impl;

import java.util.HashMap;
import java.util.Map;

import at.gv.egovernment.moa.spss.api.common.XPathFilter;

/**
 * Default implementation of <code>XPathFilter</code>.
 * 
 * @author Fatemeh Philippi
 * @version $Id$
 */
public class XPathFilterImpl implements XPathFilter {
  /** The type of filter. */
  private String filterType;
  /** The XPath expression of the filter. */
  private String xPathExpression;
  /** The namespace prefix to URI mapping to while evaluating the XPath 
   * expression. */
  private Map namespaceDeclarations = new HashMap();

  /**
   * Sets the type of filter.
   * 
   * @param filterType The type of filter.
   */
  public void setFilterType(String filterType) {
    this.filterType = filterType;
  }

  public String getFilterType() {
    return filterType;
  }

  /**
   * Sets the XPath expression of the filter.
   * 
   * @param xPathExpression The XPath expression of the filter.
   */
  public void setXPathExpression(String xPathExpression) {
    this.xPathExpression = xPathExpression;
  }

  public String getXPathExpression() {
    return xPathExpression;
  }

  /**
   * Sets the namespace prefix to URI mapping to while evaluating the XPath 
   * expression.
   * 
   * @param namespaceDeclarations The namespace prefix to URI mapping to while 
   * evaluating the XPath expression.
   */
  public void setNamespaceDelcarations(Map namespaceDeclarations) {
    this.namespaceDeclarations = namespaceDeclarations;
  }

  public Map getNamespaceDeclarations() {
    return namespaceDeclarations;
  }

}
