/*
 * Copyright 2003 Federal Chancellery Austria
 * MOA-SPSS has been developed in a cooperation between BRZ, the Federal
 * Chancellery Austria - ICT staff unit, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * http://www.osor.eu/eupl/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 */


package at.gv.egovernment.moa.spss.api.xmlsign;

import at.gv.egovernment.moa.spss.api.common.Content;

/**
 * Encapsulates information required to create a single signature.
 * 
 * @author Patrick Peck
 * @author Stephan Grill
 * @version $Id$
 */
public interface DataObjectInfo {
  /**
   * Indicates that a detached signature will be created.
   */
  public static final String STRUCTURE_DETACHED = "detached"; 
  /**
   * Indicates that an enveloping signature will be created.
   */
  public static final String STRUCTURE_ENVELOPING = "enveloping";

  /**
   * Gets the structure of the signature.
   * 
   * @return The structure of the signature.
   */
  public String getStructure();
  /**
   * Checks whether a refercence will be placed in the signature itself or
   * in the manifest.
   * 
   * @return <code>true</code> if a reference will be placed in the manifest,
   * <code>false</code> if it will be placed in the signature.
   */
  public boolean isChildOfManifest();
  /**
   * Gets information related to a single data object.
   * 
   * @return Information related to a single data object.
   */
  public Content getDataObject();
  /**
   * Gets information for the transformation of the data object.
   * 
   * @return The transformation information.
   */
  public CreateTransformsInfoProfile getCreateTransformsInfoProfile();
}
