/*
 * Copyright 2003 Federal Chancellery Austria
 * MOA-SPSS has been developed in a cooperation between BRZ, the Federal
 * Chancellery Austria - ICT staff unit, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * http://www.osor.eu/eupl/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 */


package at.gv.egovernment.moa.spss.api.xmlverify;

/**
 * Object encapsulating transform parameters either as a URI, binary or
 * hashed.
 * 
 * @author Patrick Peck
 * @author Stephan Grill
 * @version $Id$
 */
public interface TransformParameter {
  /**
   * Indicates that this object contains a transform parameter given as
   * a URI.
   */
  public static final int URI_TRANSFORMPARAMETER = 0;
  /**
   * Indicates that this object contains binary transform parameter.
   */
  public static final int BINARY_TRANSFORMPARAMETER = 1;
  /**
   * Indicatest that this object contains a binary hash of the transform
   * parameter.
   */
  public static final int HASH_TRANSFORMPARAMETER = 2;
  
  /**
   * Gets the type of transform parameter contained in this object.
   * 
   * @return The type of transform parameter, being one of 
   * <code>URI_TRANSFORMPARAMETER</code>, <code>BINARY_TRANSFORMPARAMETER</code>
   * or <code>HASH_TRANSFORMPARAMETER</code>.
   */
  public int getTransformParameterType();
  /**
   * Gets the transform parameter URI.
   * 
   * @return The transform parameter URI.
   */
  public String getURI();}
