/*
 * Copyright 2003 Federal Chancellery Austria
 * MOA-SPSS has been developed in a cooperation between BRZ, the Federal
 * Chancellery Austria - ICT staff unit, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * http://www.osor.eu/eupl/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 */


package at.gv.egovernment.moa.spss.api.xmlverify;

import java.util.List;

import at.gv.egovernment.moa.spss.api.common.CheckResult;
import at.gv.egovernment.moa.spss.api.common.SignerInfo;

/**
 * Object that encapsulates the response on a request to verify an XML
 * signature.
 * 
 * @author Patrick Peck
 * @author Stephan Grill
 * @version $Id$
 */
public interface VerifyXMLSignatureResponse {
  /**
   * Gets a <code>SignerInfo</code> element according to XMLDSig.
   * 
   * @return A <code>SignerInfo</code> element according to XMLDSig.
   */
  public SignerInfo getSignerInfo();
  /**
   * Gets datas signed by the signatory.
   * 
   * @return The signed datas.
   */
  public List getHashInputDatas();
  /**
   * Gets source datas elements.
   * 
   * @return The source datas elements.
   */
  public List getReferenceInputDatas();
  /**
  * Gets the result of the signature verification.
  * 
  * @return The result of the signature verification.
  */
  public ReferencesCheckResult getSignatureCheck();
  /**
   * Gets the result of the signature manifest verification.
   * 
   * @return The result of the signature manifest verification.
   */
  public ReferencesCheckResult getSignatureManifestCheck();
  /**
   * Gets XMLDSigManifestCheck elements.
   * 
   * @return The XMLDSigManifestCheck elements.
   */
  public List getXMLDsigManifestChecks();
  /**
   * Gets the result of the certification verification.
   * 
   * @return The result of the certificate verification.
   */
  public CheckResult getCertificateCheck();
  
  /**
   * Gets AdES Form results
   * 
   * This might be null!
   * 
   * @return The result of the AdES Form validation
   */
  public List getAdESFormResults();
  

}
