/*
 * Copyright 2003 Federal Chancellery Austria
 * MOA-SPSS has been developed in a cooperation between BRZ, the Federal
 * Chancellery Austria - ICT staff unit, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * http://www.osor.eu/eupl/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 */


package at.gv.egovernment.moa.spss.server.config;

/**
 * A class that contains information about a hardware key module.
 * 
 * @author Patrick Peck
 * @version $Id$
 */
public class HardwareKeyModule extends KeyModule {
  /** The name of the module. */
  private String name;
  /** The slod ID of the module. */
  private String slotID;
  /** The user PIN of the module. */
  private String userPIN;
  
  /**
   * Create a new <code>HardwareKey</code>.
   * 
   * @param id The key module ID.
   * @param name The name of the key.
   * @param slotID The slot ID of the key within the hardware module. May be
   * <code>null</code>.
   * @param userPIN The user PIN to access the key.
   */
  public HardwareKeyModule(String id, String name, String slotID, String userPIN) {
    super(id);
    this.name = name;
    this.slotID = slotID;
    this.userPIN = userPIN;   
  }
  
  /**
   * Return the name of this <code>HardwareKey</code>.
   * 
   * @return The name of this <code>HardwareKey</code>.
   */
  public String getName() {
    return name;
  }
  
  /**
   * Return the slot ID of this <code>HardwareKey</code>.
   * 
   * @return The slot ID of this <code>HardwareKey</code>.
   */
  public String getSlotID() {
    return slotID;
  }
  
  /**
   * Return the user PIN to access this <code>HardwareKey</code>.
   * 
   * @return The user PIN to access this <code>HardwareKey</code>.
   */
  public String getUserPIN() {
    return userPIN;
  }
}
