/*
 * Copyright 2003 Federal Chancellery Austria
 * MOA-SPSS has been developed in a cooperation between BRZ, the Federal
 * Chancellery Austria - ICT staff unit, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * http://www.osor.eu/eupl/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 */


package at.gv.egovernment.moa.spss.server.config;

import java.util.Iterator;
import java.util.Set;

/**
 * A collection of <code>KeyGroupEntry</code>s with its own ID.
 * 
 * @author Sven Aigner
 * @author Patrick Peck
 * @version $Id$
 */
public class KeyGroup {

  /** The keys belonging to this key group. */
  private Set keyGroupEntries;
  /** The key group ID. */
  private String id;
  /** The digest method algorithm for the key group */
  private String digestMethodAlgorithm;

  /**
   * Create a <code>KeyGroup</code>.
   * 
   * @param id The ID of this <code>KeyGroup</code>.
   * @param keyGroupEntries The keys belonging to this <code>KeyGroup</code>.
   * @param digestMethodAlgorithm The signature algorithm used for this key group
   */
  public KeyGroup(String id, Set keyGroupEntries, String digestMethodAlgorithm) {
    this.id = id;
    this.keyGroupEntries = keyGroupEntries;
    this.digestMethodAlgorithm = digestMethodAlgorithm; 
  }

  /**
   * Return the <code>KeyEntry</code>s contained in this <code>KeyGroup</code>.
   * 
   * @return The <code>KeyEntry</code>s contained in this <code>KeyGroup</code>.
   */
  public Set getKeyGroupEntries() {
    return keyGroupEntries;
  }
  
  /**
   * Returnd the digest method algorithm used for this key group
   * @return The digest method signature algorithm used for this key group
   */
  public String getDigestMethodAlgorithm() {
	  return digestMethodAlgorithm;
  }

  /**
   * Return the ID of this <code>KeyGroup</code>.
   * 
   * @return The <code>KeyGroup</code> ID.
   */
  public String getId() {
    return id;
  }

  /** 
   * Return a <code>String</code> representation of this <code>KeyGroup</code>.
   * 
   * @return The <code>String</code> representation.
   * @see java.lang.Object#toString()
   */
  public String toString() {
    StringBuffer sb = new StringBuffer();
    Iterator i;

    if (getKeyGroupEntries() != null) {
      i = getKeyGroupEntries().iterator();

      while (i.hasNext()) {
        sb.append(" " + i.next());
      }
    }
    return "(KeyGroup - ID:" + id + " " + sb.toString() + ")" + "DigestMethodAlgorithm: " + digestMethodAlgorithm;
  }

}
