/*
 * Copyright 2003 Federal Chancellery Austria
 * MOA-SPSS has been developed in a cooperation between BRZ, the Federal
 * Chancellery Austria - ICT staff unit, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * http://www.osor.eu/eupl/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 */


package at.gv.egovernment.moa.spss.server.config;

/**
 * A class containing information about a software key, stored in PKCS12 format.
 * 
 * @author Patrick Peck
 * @version $Id$
 */
public class SoftwareKeyModule extends KeyModule {
  /** The name of the file containing the keys. */
  private String fileName;
  /** The password for accessing the file. */
  private String passWord;
  
  /**
   * Create a new <code>SoftwareKey</code>.
   * 
   * @param id The key ID.
   * @param fileName The name of the PKCS12 keystore file containing the key.
   * @param passWord The password to access the keystore file.
   */
  public SoftwareKeyModule(String id, String fileName, String passWord) {
    super(id);
    this.fileName = fileName;
    this.passWord = passWord;
  }
  
  /**
   * Return the name of the PKCS12 keystore file containing this
   * <code>SoftwareKey</code>.
   * 
   * @return The name of the PKCS12 keystore file.
   */
  public String getFileName() {
    return fileName;
  }
  
  /**
   * Return the password to access the keystore file.
   * 
   * @return The password to access the keystore file.
   */
  public String getPassWord() {
    return passWord;
  }
  

}
