/*
 * Copyright 2003 Federal Chancellery Austria
 * MOA-SPSS has been developed in a cooperation between BRZ, the Federal
 * Chancellery Austria - ICT staff unit, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * http://www.osor.eu/eupl/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 */


package at.gv.egovernment.moa.spss.server.config;

import iaik.x509.X509Certificate;

/**
 * Information about a trust profile.
 * 
 * @author Patrick Peck
 * @version $Id$
 */
public class TrustProfile {
  /** The ID of the trust profile. */
  private String id;
  /** The URI giving the location of the trust profile. */
  private String uri;
  /** The URI giving the location of the allowed signer certificates. */
  private String signerCertsUri;
  /** Defines if Trustprofile makes use of EU TSL*/
  private boolean tslEnabled;
  /** The original URI (out of the configuration) giving the location of the trust profile (used when TSL is enabled) */
  private String uriOrig;
  /** The countries given */  
  private String countries;
  /** */
  private X509Certificate[] certificatesToBeRemoved;
  
  /**
   * Create a <code>TrustProfile</code>.
   * 
   * @param id The ID of the <code>TrustProfile</code> to create.
   * @param uri The URI of the <code>TrustProfile</code> to create.
   * @param signerCertsUri The URI of the location of the allowed signer
   *        certificates of the <code>TrustProfile</code> to create.
   */
  public TrustProfile(String id, String uri, String signerCertsUri, boolean tslEnabled, String countries) {
    this.id = id;
    this.uri = uri;
    this.signerCertsUri = signerCertsUri;
    this.tslEnabled = tslEnabled;
    this.countries = countries;
    this.certificatesToBeRemoved = new X509Certificate[0];
  }

  /**
   * Return the ID of this <code>TrustProfile</code>.
   * 
   * @return The <code>TrustProfile</code> ID.
   */
  public String getId() {
    return id;
  }

  /**
   * Return the URI of this <code>TrustProfile</code>.
   * 
   * @return The URI of <code>TrustProfile</code>.
   */
  public String getUri() {
    return uri;
  }
  
  /**
   * Return the original URI of this <code>TrustProfile</code>.
   * 
   * @return The original URI of <code>TrustProfile</code>.
   */
  public String getUriOrig() {
    return uriOrig;
  }

  /**
   * Return the URI giving the location of the allowed signer certificates
   * of this <code>TrustProfile</code>.
   * 
   * @return The URI of <code>TrustProfile</code>.
   */
  public String getSignerCertsUri() {
    return signerCertsUri;
  }
  /**
   * Returns if Trustprofile is TSL enabled
   * @return
   */
  public boolean isTSLEnabled() {
	  return tslEnabled;
  }
  /**
   * Returns the given countries
   * @return Given countries
   */
  public String getCountries() {
	  if (!tslEnabled)
		  return null;
	  else
		  return countries;
  }
  
     
  /**
   * Sets the original URI of this <code>TrustProfile</code>.
   * 
   * @return The original URI of <code>TrustProfile</code>.
   */
  public void setUriOrig(String uriOrig) {
    this.uriOrig = uriOrig;
  }
  
}
