/*
 * Copyright 2003 Federal Chancellery Austria
 * MOA-SPSS has been developed in a cooperation between BRZ, the Federal
 * Chancellery Austria - ICT staff unit, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * http://www.osor.eu/eupl/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 */


package at.gv.egovernment.moa.spss.server.iaik.config;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import iaik.pki.store.observer.NotificationData;
import iaik.pki.store.observer.Observable;
import iaik.pki.store.observer.Observer;

/**
 * A base class for observable configuration data.
 * 
 * @author Patrick Peck
 * @version $Id$
 */
public abstract class AbstractObservableConfiguration implements Observable {

  /** The observers registered with this <code>Observable</code>. */
  private List observers = new ArrayList();

  /**
   * @see iaik.pki.store.observer.Observable#addObserver(iaik.pki.store.observer.Observer)
   */
  public void addObserver(Observer observer) {
    observers.add(observer);
  }

  /**
   * @see iaik.pki.store.observer.Observable#removeObserver(iaik.pki.store.observer.Observer)
   */
  public boolean removeObserver(Observer observer) {
    return observers.remove(observer);
  }

  /**
   * @see iaik.pki.store.observer.Observable#notify(iaik.pki.store.observer.NotificationData)
   */
  public void notify(NotificationData data) {
    Iterator iter = observers.iterator();

    for (iter = observers.iterator(); iter.hasNext();) {
      Observer observer = (Observer) iter.next();
      observer.notify(data);
    }
  }
  
}
