/*
 * Copyright 2003 Federal Chancellery Austria
 * MOA-SPSS has been developed in a cooperation between BRZ, the Federal
 * Chancellery Austria - ICT staff unit, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * http://www.osor.eu/eupl/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 */


package at.gv.egovernment.moa.spss.server.iaik.config;

import java.util.Collections;
import java.util.Set;

import iaik.pki.store.certstore.CertStoreParameters;
import iaik.pki.store.certstore.CertStoreTypes;
import iaik.pki.store.certstore.directory.DirectoryCertStoreParameters;

/**
 * An implementation of the <code>DirectoryCertStoreParameters</code> interface.
 * 
 * @see iaik.pki.store.certstore.directory.DirectoryCertStoreParameters
 * @author Patrick Peck
 * @version $Id$
 */
public class DirectoryCertStoreParametersImpl
  implements DirectoryCertStoreParameters {

  /** The root directory of the <code>CertStore</code>. */
  private String rootDirectory;
  /** Whether a new directory may be created. */
  private boolean createNew;
  /** The <code>CertStore</code> ID. */
  private String id;
  /** Whether the <code>CertStore</code> is read-only. */
  private boolean readOnly;

  /**
   * Create a new <code>DirectoryCertStoreParameterImpl</code>.
   * 
   * @param id The <code>CertStore</code> ID.
   * @param rootDirectory The root directory of the <code>CertStore</code>.
   * @param createNew Whether a new directory may be created.
   * @param readOnly Whether the <code>CertStore</code> is read-only.
   */
  public DirectoryCertStoreParametersImpl(
    String id,
    String rootDirectory,
    boolean createNew,
    boolean readOnly) {
    
    this.id = id;  
    this.rootDirectory = rootDirectory;
    this.createNew = createNew;
    this.readOnly = readOnly;
  }

  /**
   * @see iaik.pki.store.certstore.directory.DirectoryCertStoreParameters#getRootDirectory()
   */
  public String getRootDirectory() {
    return rootDirectory;
  }

  /**
   * @see iaik.pki.store.certstore.directory.DirectoryCertStoreParameters#createNew()
   */
  public boolean createNew() {
    return createNew;
  }

  /**
   * @see iaik.pki.store.certstore.CertStoreParameters#getId()
   */
  public String getId() {
    return id;
  }

  /**
   * @see iaik.pki.store.certstore.CertStoreParameters#isReadOnly()
   */
  public boolean isReadOnly() {
    return readOnly;
  }

  /**
   * @return <code>CertStoreTypes.DIRECTORY</code>
   * @see iaik.pki.store.certstore.CertStoreParameters#getType()
   */
  public String getType() {
    return CertStoreTypes.DIRECTORY;
  }

@Override
public Set getVirtualStores() {
	// TODO AFITZEK IMPLEMENT THIS METHOD
	return Collections.EMPTY_SET;
}

}
