/*
 * Copyright 2003 Federal Chancellery Austria
 * MOA-SPSS has been developed in a cooperation between BRZ, the Federal
 * Chancellery Austria - ICT staff unit, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * http://www.osor.eu/eupl/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 */


package at.gv.egovernment.moa.spss.server.iaik.config;

import iaik.server.modules.keys.HardwareKeyModuleConfiguration;

import at.gv.egovernment.moa.spss.server.config.HardwareKeyModule;

/**
 * An implementation of the <code>HardwareKeyModuleConfiguration</code>
 * interface wrapping a <code>HardwareKeyModule</code> from the MOA
 * configuration.
 * 
 * @see iaik.server.modules.keys.HardwareKeyModuleConfiguration
 * @author Patrick Peck
 * @version $Id$
 */
public class HardwareKeyModuleConfigurationImpl
  extends AbstractKeyModuleConfigurationImpl
  implements HardwareKeyModuleConfiguration {

  /** The wrapped <code>HardwareKeyModule</code>. */
  private HardwareKeyModule keyModule;

  /**
   * Create a new <code>HardwareKeyModuleConfigurationImpl</code>.
   * 
   * @param keyModule The <code>HardwareKeyModule</code> from the underlying
   * MOA configuration.
   */
  public HardwareKeyModuleConfigurationImpl(HardwareKeyModule keyModule) {
    super(keyModule.getId());
    this.keyModule = keyModule;
  }

  /**
   * @see iaik.server.modules.keys.HardwareKeyModuleConfiguration#getModuleName()
   */
  public String getModuleName() {
    return keyModule.getName();
  }

  /**
   * @see iaik.server.modules.keys.HardwareKeyModuleConfiguration#getSlotID()
   */
  public String getSlotID() {
    return keyModule.getSlotID();
  }

  /**
   * @see iaik.server.modules.keys.HardwareKeyModuleConfiguration#getUserPIN()
   */
  public char[] getUserPIN() {
    return keyModule.getUserPIN().toCharArray();
  }

}
