/*
 * Copyright 2003 Federal Chancellery Austria
 * MOA-SPSS has been developed in a cooperation between BRZ, the Federal
 * Chancellery Austria - ICT staff unit, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * http://www.osor.eu/eupl/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 */


package at.gv.egovernment.moa.spss.server.iaik.config;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;

import iaik.server.modules.keys.ConfigurationException;
import iaik.server.modules.keys.SoftwareKeyModuleConfiguration;
import at.gv.egovernment.moa.spss.server.config.SoftwareKeyModule;
import at.gv.egovernment.moa.spss.util.MessageProvider;
import at.gv.egovernment.moaspss.logging.LogMsg;
import at.gv.egovernment.moaspss.logging.Logger;

/**
 * An implementation of the <code>SoftwareKeyModuleConfiguration</code> wrapping
 * a <code>SoftwareKeyModule</code> from the MOA configuration.
 * 
 * @see iaik.server.modules.keys.SoftwareKeyModuleConfiguration
 * @author Patrick Peck
 * @version $Id$
 */
public class SoftwareKeyModuleConfigurationImpl
  extends AbstractKeyModuleConfigurationImpl
  implements SoftwareKeyModuleConfiguration {

  /** The wrapped <code>SoftwareKeyModule</code>. */
  private SoftwareKeyModule keyModule;

  /**
   * Create a new <code>SoftwareKeyModuleConfigurationImpl</code>.
   * 
   * @param keyModule The <code>SoftwareKeyModule</code> from the underlying MOA
   * configuration.
   */
  public SoftwareKeyModuleConfigurationImpl(SoftwareKeyModule keyModule) {
    super(keyModule.getId());
    this.keyModule = keyModule;
  }

  /**
   * @see iaik.server.modules.keys.SoftwareKeyModuleConfiguration#getKeyStoreTypeName()
   */
  public String getKeyStoreTypeName() {
    return KEY_STORE_TYPE_NAME_PKCS12;
  }

  /**
   * @see iaik.server.modules.keys.SoftwareKeyModuleConfiguration#getKeyStoreAsStream()
   */
  public InputStream getKeyStoreAsStream() {
    MessageProvider msg = MessageProvider.getInstance();

    try {
      String message = 
        msg.getMessage("config.18", new Object[] { keyModule.getFileName()});
      Logger.info(new LogMsg(message));
      return new FileInputStream(keyModule.getFileName());
    } catch (FileNotFoundException e) {
      String message =
        msg.getMessage("config.09", new Object[] { keyModule.getFileName()});

      throw new ConfigurationException(message, e, null);
    }
  }

  /**
   * @see iaik.server.modules.keys.SoftwareKeyModuleConfiguration#getKeyStoreAuthenticationData()
   */
  public char[] getKeyStoreAuthenticationData() {
    return keyModule.getPassWord().toCharArray();
  }

}
