/*
 * Copyright 2003 Federal Chancellery Austria
 * MOA-SPSS has been developed in a cooperation between BRZ, the Federal
 * Chancellery Austria - ICT staff unit, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * http://www.osor.eu/eupl/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 */


package at.gv.egovernment.moa.spss.server.iaik.config;

import java.security.cert.X509Certificate;
import java.security.spec.AlgorithmParameterSpec;

import iaik.pki.pathvalidation.ValidationConfiguration;

import at.gv.egovernment.moa.spss.server.config.ConfigurationProvider;

/**
 * An implementation of the <code>ValidationConfiguration</code> interface using
 * MOA configuration data.
 * 
 * @see iaik.pki.pathvalidation.ValidationConfiguration
 * @author Patrick Peck
 * @version $Id$
 */
public class ValidationConfigurationImpl
  extends AbstractObservableConfiguration
  implements ValidationConfiguration {

  /** The <code>ConfigurationProvider</code> to read the configuration data
   * from. */
  private ConfigurationProvider config;

  /**
   * Create a new <code>ValidationConfigurationImpl</code>.
   * 
   * @param config The underlying MOA configuration data.
   */
  public ValidationConfigurationImpl(ConfigurationProvider config) {
    this.config = config;
  }

  /**
   * @see iaik.pki.pathvalidation.ValidationConfiguration#getChainingMode(java.security.cert.X509Certificate)
   */
  public String getChainingMode(X509Certificate cert) {
    return config.getChainingMode(cert);
  }

  /**
   * @see iaik.pki.pathvalidation.ValidationConfiguration#getPublicKeyParamsAsSpec(java.security.cert.X509Certificate)
   */
  public AlgorithmParameterSpec getPublicKeyParamsAsSpec(X509Certificate cert) {
    return null;
  }

  /**
   * @see iaik.pki.pathvalidation.ValidationConfiguration#getPublicKeyParamsAsCert(java.security.cert.X509Certificate)
   */
  public X509Certificate getPublicKeyParamsAsCert(X509Certificate cert) {
    return null;
  }

}
