/*
 * Copyright 2003 Federal Chancellery Austria
 * MOA-SPSS has been developed in a cooperation between BRZ, the Federal
 * Chancellery Austria - ICT staff unit, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * http://www.osor.eu/eupl/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 */


package at.gv.egovernment.moa.spss.server.iaik.pki.pathvalidation;

import iaik.pki.pathvalidation.ValidationProfile;

import java.util.Collections;
import java.util.Set;

import at.gv.egovernment.moa.spss.server.config.ConfigurationProvider;

/**
 * An implementation of the <code>ValidationProfile</code> interface providing
 * information about certificat path validation. 
 * 
 * @author Patrick Peck
 * @version $Id$
 */
public class ValidationProfileImpl implements ValidationProfile {

  /** The <code>ConfigurationProvider</code> to read the configuration data
   * from. */
  private ConfigurationProvider config;
  private boolean initialAnyPolicyInhibit;
  private boolean initialExplicitPolicy;
  private boolean initialPolicyMappingInhibit;
  private Set initialPolicySet;
  private boolean nameConstraintsProcessing;
  private boolean policyProcessing;

  /**
   * Create a new <code>ValidationProfileImpl</code> object.
   * 
   * This objects's fields are preset to the following values:
   * 
   * <ul>
   * <li><code>initialAnyPolicyInhibit = true</code></li>
   * <li><code>initialExplicitPoliy = true</code></li>
   * <li><code>initialPolicyMappingInhibit = true</code></li>
   * <li><code>initialPolicySet = empty</code></li>
   * <li><code>policyProcessing = false</code></li>
   * <li><code>nameConstraintsProcessing = false</code></li>
   * <li><code>revocationChecking = false</code></li>
   * </ul>
   * 
   * @param config MOA configuration data for additional configuration
   * information (currently unused). 
   */
  public ValidationProfileImpl(ConfigurationProvider config) {
    this.config = config;
    initialAnyPolicyInhibit = true;
    initialExplicitPolicy = true;
    initialPolicyMappingInhibit = true;
    initialPolicySet = Collections.EMPTY_SET;
    policyProcessing = false;
    nameConstraintsProcessing = false;
  }

  /**
   * @see iaik.pki.pathvalidation.ValidationProfile#getInitialAnyPolicyInhibit()
   */
  public boolean getInitialAnyPolicyInhibit() {
    return initialAnyPolicyInhibit;
  }

  /**
   * @see iaik.pki.pathvalidation.ValidationProfile#getInitialExplicitPolicy()
   */
  public boolean getInitialExplicitPolicy() {
    return initialExplicitPolicy;
  }

  /**
   * @see iaik.pki.pathvalidation.ValidationProfile#getInitialPolicyMappingInhibit()
   */
  public boolean getInitialPolicyMappingInhibit() {
    return initialPolicyMappingInhibit;
  }

  /**
   * @see iaik.pki.pathvalidation.ValidationProfile#getInitialPolicySet()
   */
  public Set getInitialPolicySet() {
    return initialPolicySet;
  }

  /**
   * @see iaik.pki.pathvalidation.ValidationProfile#getPolicyProcessing()
   */
  public boolean getPolicyProcessing() {
    return policyProcessing;
  }

  /**
   * @see iaik.pki.pathvalidation.ValidationProfile#getNameConstraintsProcessing()
   */
  public boolean getNameConstraintsProcessing() {
    return nameConstraintsProcessing;
  }

  /**
   * @see iaik.pki.pathvalidation.ValidationProfile#getRevocationChecking()
   */
  public boolean getRevocationChecking() 
  {
    return config.getEnableRevocationChecking();
  }

}