/*
 * Copyright 2003 Federal Chancellery Austria
 * MOA-SPSS has been developed in a cooperation between BRZ, the Federal
 * Chancellery Austria - ICT staff unit, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * http://www.osor.eu/eupl/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 */


package at.gv.egovernment.moa.spss.server.iaik.pki.revocation;

import java.security.cert.X509Certificate;

import iaik.pki.revocation.RevocationProfile;
import iaik.pki.revocation.RevocationSourceTypes;

import at.gv.egovernment.moa.spss.server.config.ConfigurationProvider;

/**
 * An implementation of the <code>RevocationProfile</code> interface providing
 * information about revocation status checking, based on MOA configuration
 * data.
 * 
 * @author Patrick Peck
 * @version $Id$
 */
public class RevocationProfileImpl implements RevocationProfile {
  /** The default service order. */
  private static final String[] DEFAULT_SERVICE_ORDER =
    { RevocationSourceTypes.CRL, RevocationSourceTypes.OCSP };
  /** The <code>ConfigurationProvider</code> to read the MOA configuration data
   * from. */
  private ConfigurationProvider config;
  /** The OCSP request hash algorithm. Currently only "SHA" is supported. */
  private static final String oCSPRequestHashAlgorithm = "SHA";

  /**
   * Create a new <code>RevocationProfileImpl</code>.
   * 
   * @param config The MOA configuration data.
   */
  public RevocationProfileImpl(ConfigurationProvider config) {
    this.config = config;
    // currently only "SHA" is supported
//    this.oCSPRequestHashAlgorithm = "";
  }

  /**
   * @see iaik.pki.revocation.RevocationProfile#getMaxRevocationAge(String)
   */
  public long getMaxRevocationAge(String distributionPointUri) 
  {
    return config.getMaxRevocationAge();
  }

  /**
   * @see iaik.pki.revocation.RevocationProfile#getOCSPRequestHashAlgorithm()
   */
  public String getOCSPRequestHashAlgorithm() {
    return oCSPRequestHashAlgorithm;
  }

  /**
   * @see iaik.pki.revocation.RevocationProfile#getPreferredServiceOrder(java.security.cert.X509Certificate)
   */
  public String[] getPreferredServiceOrder(X509Certificate cert) 
  {
    String[] serviceOrder = config.getServiceOrder();
    if (serviceOrder == null || serviceOrder.length == 0) return DEFAULT_SERVICE_ORDER; 
    return serviceOrder;
  }

}
