/*
 * Copyright 2003 Federal Chancellery Austria
 * MOA-SPSS has been developed in a cooperation between BRZ, the Federal
 * Chancellery Austria - ICT staff unit, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * http://www.osor.eu/eupl/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 */


package at.gv.egovernment.moa.spss.server.iaik.xml;

import java.io.ByteArrayInputStream;
import java.io.InputStream;

import iaik.server.modules.xml.BinaryDataObject;

/**
 * A <code>BinaryDataObject</code> encapsulating Base64 data.
 * 
 * @author Patrick Peck
 * @version $Id$
 */
public class ByteArrayDataObjectImpl
  extends DataObjectImpl
  implements BinaryDataObject {

  /** The binary data contained in this <code>BinaryDataObject</code>. */
  private byte[] bytes;

  /**
   * Create a new <code>ByteArrayDataObjectImpl</code>.
   * 
   * @param bytes The binary data contained in this 
   * <code>BinaryDataObject</code>.
   */
  public ByteArrayDataObjectImpl(byte[] bytes) {
    setBytes(bytes);
  }

  /**
   * Set the Base64 data.
   * 
   * @param bytes The binary data contained in this 
   * <code>BinaryDataObject</code>.
   */
  public void setBytes(byte[] bytes) {
    this.bytes = bytes;
  }

  /**
   * Return the binary data encoded in the Base64 <code>String</code> as a
   * stream.
   * 
   * @return The binary data contained in this object, as a
   * <code>InputStream</code>. Repeated calls to this function will return a
   * new stream to the Base64 data.
   * @see iaik.server.modules.xml.BinaryDataObject#getInputStream()
   */
  public InputStream getInputStream() {
    return new ByteArrayInputStream(bytes);
  }

}
