/*
 * Copyright 2003 Federal Chancellery Austria
 * MOA-SPSS has been developed in a cooperation between BRZ, the Federal
 * Chancellery Austria - ICT staff unit, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * http://www.osor.eu/eupl/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 */


package at.gv.egovernment.moa.spss.server.iaik.xml;

import java.io.InputStream;

import iaik.server.modules.xml.BinaryDataObject;

/**
 * A <code>BinaryDataObject</code> encapsulating binary data from a stream. 
 * 
 * @author Patrick Peck
 * @version $Id$
 */
public class ByteStreamDataObjectImpl
  extends DataObjectImpl
  implements BinaryDataObject {

  /** The <code>InputStream</code> containing the binary data. */
  private InputStream inputStream;

  /**
   * Create a new <code>ByteStreamDataObjectImpl</code>.
   * 
   * @param inputStream The stream from which to read the binary data.
   */
  public ByteStreamDataObjectImpl(InputStream inputStream) {
    setInputStream(inputStream);
  }

  /**
   * Set the input stream from which to read the binary data.
   * 
   * @param inputStream The input stream from which to read the binary data.
   */
  public void setInputStream(InputStream inputStream) {
    this.inputStream = inputStream;
  }

  /**
   * Return the binary data from this object as a stream.
   * 
   * @return The stream containing the binary data. Calling this function
   * repeatedly will always return the same <code>InputStream</code>.
   * @see iaik.server.modules.xml.BinaryDataObject#getInputStream()
   */
  public InputStream getInputStream() {
    return inputStream;
  }

}
