/*
 * Copyright 2003 Federal Chancellery Austria
 * MOA-SPSS has been developed in a cooperation between BRZ, the Federal
 * Chancellery Austria - ICT staff unit, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * http://www.osor.eu/eupl/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 */


package at.gv.egovernment.moa.spss.server.iaik.xml;

import iaik.server.modules.xml.Transformation;

/**
 * Base implementation class for <code>Transformation</code> derived classes.
 * 
 * @author Patrick Peck
 * @version $Id$
 */
public abstract class TransformationImpl implements Transformation {

  /** The algorithm URI identifying the transformation algorithm. */
  private String algorithmURI;
  
  /**
   * @see iaik.server.modules.xml.Transformation#getAlgorithmURI()
   */
  public String getAlgorithmURI() {
    return algorithmURI;
  }

  /**
   * Sets the algorithm URI.
   * 
   * @param algorithmURI The algorithm URI to set.
   */
  protected void setAlgorithmURI(String algorithmURI) {
    this.algorithmURI = algorithmURI;
  }
  
  /**
   * Returns the hash code of the algorithm URI. Should be overridden if a
   * transformation distinguishes itself from others by more than just the
   * algorithm URI.
   * 
   * @see java.lang.Object#hashCode()
   */
  public int hashCode() {
    return getAlgorithmURI().hashCode();
  }
  
}
