/*
 * Copyright 2003 Federal Chancellery Austria
 * MOA-SPSS has been developed in a cooperation between BRZ, the Federal
 * Chancellery Austria - ICT staff unit, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * http://www.osor.eu/eupl/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 */


package at.gv.egovernment.moa.spss.server.iaik.xml;

import org.w3c.dom.Element;

import iaik.server.modules.xml.XMLSignature;

/**
 * An object containing an XMLDsig signature in the form of a 
 * <code>dsig:Signature</code> DOM element.
 * 
 * @author Patrick Peck
 * @version $Id$
 */
public class XMLSignatureImpl implements XMLSignature {
  /** The signature DOM element. */
  private Element element;
  
  /**
   * Create a new <code>XMLSignatureImpl</code>.
   * 
   * @param element The <code>dsig:Signature</code> DOM element.
   */
  public XMLSignatureImpl(Element element) {
    setElement(element);
  }
  
  /**
   * Set the <code>dsig:Signature</code> DOM element.
   * 
   * @param element The <code>dsig:Signature</code> element to set.
   */
  public void setElement(Element element) {
    this.element = element;
  }

  /**
   * @see iaik.server.modules.xml.XMLSignature#getElement()
   */
  public Element getElement() {
    return element;
  }

}
