/*
 * Copyright 2003 Federal Chancellery Austria
 * MOA-SPSS has been developed in a cooperation between BRZ, the Federal
 * Chancellery Austria - ICT staff unit, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * http://www.osor.eu/eupl/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 */


package at.gv.egovernment.moa.spss.server.iaik.xmlsign;

import iaik.server.modules.xmlsign.XMLSignatureInsertionLocation;

/**
 * An object giving the location of where the signature will be
 * inserted into the parent element.
 * 
 * @author Patrick Peck
 * @version $Id$
 */
public class XMLSignatureInsertionLocationImpl
  implements XMLSignatureInsertionLocation {

  /** Where to put the signature into the signature parent element. */
  private int signatureChildIndex;
  
  /**
   * Create a new <code>XMLSignatureInsertLocationImpl</code>.
   * 
   * @param signatureChildIndex The position index at which to append the
   * signature to the parent element.
   */
  public XMLSignatureInsertionLocationImpl(int signatureChildIndex) {
    setSignatureChildIndex(signatureChildIndex);
  }

  /**
   * @see iaik.server.modules.xmlsign.XMLSignatureInsertionLocation#getSignatureChildIndex()
   */
  public int getSignatureChildIndex() {
    return signatureChildIndex;
  }

  /**
   * Sets the position index at which to append the signature to the parent
   * element.
   * 
   * @param signatureChildIndex The position index to set.
   */
  public void setSignatureChildIndex(int signatureChildIndex) {
    this.signatureChildIndex = signatureChildIndex;
  }

}
