/*
 * Copyright 2003 Federal Chancellery Austria
 * MOA-SPSS has been developed in a cooperation between BRZ, the Federal
 * Chancellery Austria - ICT staff unit, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * http://www.osor.eu/eupl/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 */


package at.gv.egovernment.moa.spss.server.invoke;

import at.gv.egovernment.moa.spss.MOAException;
import at.gv.egovernment.moa.spss.api.cmsverify.VerifyCMSSignatureRequest;
import at.gv.egovernment.moa.spss.server.config.ConfigurationProvider;
import at.gv.egovernment.moa.spss.server.iaik.cmsverify.CMSSignatureVerificationProfileImpl;
import at.gv.egovernment.moa.spss.server.iaik.cmsverify.PDFSignatureVerificationProfileImpl;
import at.gv.egovernment.moa.spss.server.iaik.pki.PKIProfileImpl;
import at.gv.egovernment.moa.spss.server.transaction.TransactionContext;
import at.gv.egovernment.moa.spss.server.transaction.TransactionContextManager;
import iaik.server.modules.cmsverify.CMSSignatureVerificationProfile;
import iaik.server.modules.pdfverify.PDFSignatureVerificationProfile;

/**
 * A factory to create a <code>CMSSignatureVerificationProfile</code> from a
 * <code>VerifyCMSSignatureRequest</code> and the current MOA configuration
 * data.
 * 
 * @author Patrick Peck
 * @version $Id$
 */
public class CMSSignatureVerificationProfileFactory {

  /** The <code>VerifyCMSSignatureRequest</code> to draw profile data from. */  
  private VerifyCMSSignatureRequest request;

  /**
   * Create a new <code>CMSSignatureVerificationProfileFactory</code>.
   *
   * @param request The <code>VerifyCMSSignatureRequest</code> to draw profile 
   * data from.
   */
  public CMSSignatureVerificationProfileFactory(VerifyCMSSignatureRequest request) {
    this.request = request;
  }

  /**
   * Create a <code>CMSSignatureVerificationProfile</code> from the given
   * request and the current MOA configuration.
   * 
   * @return The <code>CMSSignatureVerificationProfile</code> for the
   * <code>request</code>, based on the current configuration.
   * @throws MOAException An error occurred creating the profile.
   */
  public PDFSignatureVerificationProfile createPDFProfile()
    throws MOAException {
    TransactionContext context =
      TransactionContextManager.getInstance().getTransactionContext();
    ConfigurationProvider config = context.getConfiguration();
    PDFSignatureVerificationProfileImpl profile =
      new PDFSignatureVerificationProfileImpl();
    String trustProfileID;

    // set the certificate validation profile
    trustProfileID = request.getTrustProfileId();
    profile.setCertificateValidationProfile(
      new PKIProfileImpl(config, trustProfileID));

    return profile;
  }
  
  /**
   * Create a <code>CMSSignatureVerificationProfile</code> from the given
   * request and the current MOA configuration.
   * 
   * @return The <code>CMSSignatureVerificationProfile</code> for the
   * <code>request</code>, based on the current configuration.
   * @throws MOAException An error occurred creating the profile.
   */
  public CMSSignatureVerificationProfile createProfile()
    throws MOAException {
    TransactionContext context =
      TransactionContextManager.getInstance().getTransactionContext();
    ConfigurationProvider config = context.getConfiguration();
    CMSSignatureVerificationProfileImpl profile =
      new CMSSignatureVerificationProfileImpl();
    String trustProfileID;

    // set the certificate validation profile
    trustProfileID = request.getTrustProfileId();
    profile.setCertificateValidationProfile(
      new PKIProfileImpl(config, trustProfileID));

    return profile;
  }

}
