/*
 * Copyright 2003 Federal Chancellery Austria
 * MOA-SPSS has been developed in a cooperation between BRZ, the Federal
 * Chancellery Austria - ICT staff unit, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * http://www.osor.eu/eupl/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 */


package at.gv.egovernment.moa.spss.server.invoke;

import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import at.gv.egovernment.moa.spss.MOAApplicationException;
import at.gv.egovernment.moa.spss.api.common.ElementSelector;
import at.gv.egovernment.moaspss.util.XPathException;
import at.gv.egovernment.moaspss.util.XPathUtils;

/**
 * Utility methods for invoking the IAIK MOA modules.
 * 
 * @author Patrick Peck
 * @version $Id$
 */
public class InvokerUtils {

  /**
   * Select the signature parent element.
   * 
   * @param root The root DOM element which contains the signature parent
   * element somewhere in its subtree.
   * @param location The <code>ElementSelector</code> containing the XPath 
   * expression to select the signature parent element from the document. 
   * It is also contains the namespace prefix to URI mapping.
   * @return Element The signature parent element.
   * @throws MOAApplicationException An error occurred evaluating the
   * <code>location</code>.
   */
  public static Element evaluateSignatureLocation(
    Element root,
    ElementSelector location)
    throws MOAApplicationException {

    NodeList nodes;

    try {
      nodes =
        XPathUtils.selectNodeList(
          root,
          location.getNamespaceDeclarations(),
          location.getXPathExpression());
    } catch (XPathException e) {
      throw new MOAApplicationException(
        "2212",
        new Object[] { location.getXPathExpression()},
        e);
    }

    if (nodes.getLength() != 1
      || !(nodes.item(0).getNodeType() == Node.ELEMENT_NODE)) {
      throw new MOAApplicationException(
        "2212",
        new Object[] { location.getXPathExpression()});
    }
    return (Element) nodes.item(0);
  }


}
