/*
 * Copyright 2003 Federal Chancellery Austria
 * MOA-SPSS has been developed in a cooperation between BRZ, the Federal
 * Chancellery Austria - ICT staff unit, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * http://www.osor.eu/eupl/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 */


package at.gv.egovernment.moa.spss.server.invoke;

import java.util.Collections;

import at.gv.egovernment.moa.spss.MOAException;
import at.gv.egovernment.moa.spss.api.Configurator;
import at.gv.egovernment.moa.spss.api.SignatureCreationService;
import at.gv.egovernment.moa.spss.api.xmlsign.CreateXMLSignatureRequest;
import at.gv.egovernment.moa.spss.api.xmlsign.CreateXMLSignatureResponse;

/**
 * An implementation of the <code>SignatureCreationService</code>, using
 * the <code>XMLSignatureCreationInvoker</code>.
 * 
 * @author Patrick Peck
 * @version $Id$
 */
public class SignatureCreationServiceImpl extends SignatureCreationService {

  /**
   * Create an XML signature.
   * 
   * @param request The <code>CreateXMLSignatureRequest</code> containing
   * information about the signature(s) to create.
   * @return The created signature(s).
   * @throws MOAException An error occurred creating the signature(s).
   */
  public CreateXMLSignatureResponse createXMLSignature(CreateXMLSignatureRequest request)
    throws MOAException {

    XMLSignatureCreationInvoker invoker =
      XMLSignatureCreationInvoker.getInstance();
    CreateXMLSignatureResponse response;

    try {
    	
      Configurator.getInstance().init();
      ServiceContextUtils.setUpContexts();
      response = invoker.createXMLSignature(request, Collections.EMPTY_SET);
      
      return response;
    } finally {
      ServiceContextUtils.tearDownContexts();
    }
  }

}
