/*
 * Copyright 2003 Federal Chancellery Austria
 * MOA-SPSS has been developed in a cooperation between BRZ, the Federal
 * Chancellery Austria - ICT staff unit, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * http://www.osor.eu/eupl/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 */


package at.gv.egovernment.moa.spss.server.invoke;

import at.gv.egovernment.moa.spss.MOAException;
import at.gv.egovernment.moa.spss.api.Configurator;
import at.gv.egovernment.moa.spss.api.SignatureVerificationService;
import at.gv.egovernment.moa.spss.api.cmsverify.VerifyCMSSignatureRequest;
import at.gv.egovernment.moa.spss.api.cmsverify.VerifyCMSSignatureResponse;
import at.gv.egovernment.moa.spss.api.xmlverify.VerifyXMLSignatureRequest;
import at.gv.egovernment.moa.spss.api.xmlverify.VerifyXMLSignatureResponse;

/**
 * An implementation of the <code>SignatureVerificationService</code> using
 * the <code>XMLSignatureVerificationInvoker</code> and the
 * <code>CMSSignatureVerificationInvoker</code>.
 * 
 * @author Patrick Peck
 * @version $Id$
 */
public class SignatureVerificationServiceImpl
  extends SignatureVerificationService {

  /**
   * Verify a CMS signature.
   * 
   * @param request The <code>VerifyCMSSignatureRequest</code> containing 
   * information about the signature verification.
   * @return The result of the signature verification.
   * @throws MOAException An error occurred during signature verification.
   */
  public VerifyCMSSignatureResponse verifyCMSSignature(VerifyCMSSignatureRequest request)
    throws MOAException {

    CMSSignatureVerificationInvoker invoker =
      CMSSignatureVerificationInvoker.getInstance();
    VerifyCMSSignatureResponse response;

    try {
      Configurator.getInstance().init();
      ServiceContextUtils.setUpContexts();
      response = invoker.verifyCMSSignature(request);
      
      return response;
    } finally {
      ServiceContextUtils.tearDownContexts();
    }
  }

  /**
   * Verify an XML signature.
   * 
   * @param request The <code>VerifyXMLSignatureRequest</code> containinig
   * information about the signature verification.
   * @return The result of the signature verification.
   * @throws MOAException An error occurred during signature verification.
   */
  public VerifyXMLSignatureResponse verifyXMLSignature(VerifyXMLSignatureRequest request)
    throws MOAException {

    XMLSignatureVerificationInvoker invoker =
      XMLSignatureVerificationInvoker.getInstance();
    VerifyXMLSignatureResponse response;

    try {
    	

      Configurator.getInstance().init();
      ServiceContextUtils.setUpContexts();
      response = invoker.verifyXMLSignature(request);
            
      return response;
    } finally {
      ServiceContextUtils.tearDownContexts();
    }
  }

}
