/*
 * Copyright 2003 Federal Chancellery Austria
 * MOA-SPSS has been developed in a cooperation between BRZ, the Federal
 * Chancellery Austria - ICT staff unit, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * http://www.osor.eu/eupl/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 */


package at.gv.egovernment.moa.spss.server.logging;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import iaik.logging.TransactionId;

/**
 * An implementation of the <code>iaik.logging.Log</code>
 * interface that is based on Jakarta Commons-Logging.
 * 
 * @author Fatemeh Philippi
 * @version $Id$
 */
public class IaikLog implements iaik.logging.Log {
  /** The hierarchy to log all IAIK output to. */
  public static final String IAIK_LOG_HIERARCHY = "iaik.server";
  /** The commons-loggin <code>Log</code> to use for logging the messages. */
  private static Logger log = LoggerFactory.getLogger(IAIK_LOG_HIERARCHY);
  /** The node ID to use. */
  private String nodeId;
  
  /**
   * Create a new <code>IaikLog</code>.
   * 
   * @param nodeId The node ID for this <code>Log</code> object. 
   */
  public IaikLog(String nodeId) {
    this.nodeId = nodeId;
  }

  /**
   * @see iaik.logging.Log#isDebugEnabled()
   */
  public boolean isDebugEnabled() {
    return log.isDebugEnabled();
  }

  /**
   * @see iaik.logging.Log#debug(TransactionId, Object, Throwable)
   */
  public void debug(TransactionId transactionId, Object message, Throwable t) {
    IaikLogMsg msg = new IaikLogMsg(transactionId, nodeId, message);

    log.debug(msg.toString(), t);
  }

  /**
   * @see iaik.logging.Log#isInfoEnabled()
   */
  public boolean isInfoEnabled() {
    return log.isInfoEnabled();
  }

  /**
   * @see iaik.logging.Log#info(TransactionId, Object, Throwable)
   */
  public void info(TransactionId transactionId, Object message, Throwable t) {
    IaikLogMsg msg = new IaikLogMsg(transactionId, nodeId, message);

    log.info(msg.toString(), t);
  }

  /**
   * @see iaik.logging.Log#isWarnEnabled()
   */
  public boolean isWarnEnabled() {
    return log.isWarnEnabled();
  }

  /**
   * @see iaik.logging.Log#warn(TransactionId, Object, Throwable)
   */
  public void warn(TransactionId transactionId, Object message, Throwable t) {
    IaikLogMsg msg = new IaikLogMsg(transactionId, nodeId, message);

    log.warn(msg.toString(), t);
  }

  /**
   * @see iaik.logging.Log#isErrorEnabled()
   */
  public boolean isErrorEnabled() {
    return log.isErrorEnabled();
  }

  /**
   * @see iaik.logging.Log#error(TransactionId, Object, Throwable)
   */
  public void error(TransactionId transactionId, Object message, Throwable t) {
    IaikLogMsg msg = new IaikLogMsg(transactionId, nodeId, message);

    log.error(msg.toString(), t);
  }

  /**
   * @see iaik.logging.Log#isFatalEnabled()
   */
  public boolean isFatalEnabled() {
    return log.isErrorEnabled();
  }

  /**
   * @see iaik.logging.Log#fatal(TransactionId, Object, Throwable)
   */
  public void fatal(TransactionId transactionId, Object message, Throwable t) {
    IaikLogMsg msg = new IaikLogMsg(transactionId, nodeId, message);

    log.error(msg.toString(), t);
  }

  /**
   * @see iaik.logging.Log#setNodeId(String)
   */
  public void setNodeId(String nodeId) {
    this.nodeId = nodeId;
  }

  /**
   * @see iaik.logging.Log#getNodeId()
   */
  public String getNodeId() {
    return nodeId;
  }

}
