/*
 * Copyright 2003 Federal Chancellery Austria
 * MOA-SPSS has been developed in a cooperation between BRZ, the Federal
 * Chancellery Austria - ICT staff unit, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * http://www.osor.eu/eupl/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 */


package at.gv.egovernment.moa.spss.server.logging;

import at.gv.egovernment.moaspss.logging.LoggingContextManager;
import iaik.logging.Log;
import iaik.logging.LogConfigurationException;
import iaik.logging.LogFactory;

/**
 * An implementation of the <code>iaik.logging.LogFactory</code> abstract
 * class to log messages to the MOA logging subsystem.
 * 
 * @author Patrick Peck
 * @version $Id$
 */
public class IaikLogFactory extends LogFactory {

  public Log getInstance(Class clazz) throws LogConfigurationException {
    return getInstanceImpl();
  }

  public Log getInstance(String name) throws LogConfigurationException {
    return getInstanceImpl();
  }

  /**
   * Return an instance of <code>iaik.logging.Log</code>.
   * 
   * @return The <code>iaik.logging.Log</code> object to log messages to.
   */  
  private Log getInstanceImpl() {
    String nodeID = 
      LoggingContextManager.getInstance().getLoggingContext().getNodeID();
      
    return new IaikLog(nodeID);
  }

  public void release() {
    // we do not hold any resources
  }

}
