/*
 * Copyright 2003 Federal Chancellery Austria
 * MOA-SPSS has been developed in a cooperation between BRZ, the Federal
 * Chancellery Austria - ICT staff unit, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * http://www.osor.eu/eupl/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 */


package at.gv.egovernment.moa.spss.server.logging;

import iaik.logging.TransactionId;

/**
 * A unified message type to log messages from the IAIK subsystem.
 * 
 * @author Patrick Peck
 * @version $Id$
 */
public class IaikLogMsg {
  
  /** The transaction ID of this message. */
  private TransactionId transactionId;
  /** The node ID of this message. */
  private String nodeId;
  /** The message to log. */
  private Object message;
  
  /**
   * Create a <code>IaikLogMsg</code> object.
   * 
   * @param transactionId The transaction id of the transaction which
   * generated this log message. May be <code>null</code>.
   * @param nodeId The node id where this message was generated. May be
   * <code>null</code>.
   * @param message The actual message to log. May be <code>null</code>.
   */
  public IaikLogMsg(TransactionId transactionId, String nodeId, Object message) {
    this.transactionId = transactionId;
    this.nodeId = nodeId;
    this.message = message;
  }

  
  /**
   * Convert this log message to a <code>String</code>.
   * 
   * @return The <code>String</code> representation of this log message. 
   */
  public String toString() {
    StringBuffer msg = new StringBuffer();
    
    msg.append("TID=");
    msg.append(transactionId != null ? transactionId.getLogID() : "<null>");
    msg.append(" NID=");
    msg.append(nodeId != null ? nodeId : "<null>");
    msg.append(" MSG=");
    msg.append(message != null ? message.toString() : "<null>");
    
    return msg.toString();
  }

}
