/*
 * Copyright 2003 Federal Chancellery Austria
 * MOA-SPSS has been developed in a cooperation between BRZ, the Federal
 * Chancellery Austria - ICT staff unit, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * http://www.osor.eu/eupl/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 */


package at.gv.egovernment.moa.spss.server.util;

import java.util.Set;

/**
 * Generate unique ID values for various objects in the response.
 * 
 * @author Patrick Peck
 * @version $Id$
 */
public class IdGenerator {
  /** The base value to append the counter to. */
  private String base;
  /** The <code>Set</code> of reserved ID values. */
  private Set reserved;
  /** The sequence number. */
  private int count;
  
  /**
   * Create a new <code>IdGenerator</code>.
   * 
   * @param base A base value to append the IDs to. The creator of this object
   * should provide a base value, so that appending the counter leads to unique
   * IDs.
   * @param reserved The <code>Set</code> of reserved IDs. A call to 
   * <code>uniqueId()</code> will respect the reserved IDs.
   */
  public IdGenerator(String base, Set reserved) {
    this.base = base;
    this.reserved = reserved;
    count = 1;
  }
  
  /**
   * Create the next ID value in the sequence.
   * 
   * @return The next ID value in the sequence.
   */
  protected String nextId() {
    return base + "-" + count++;
  }
  
  /**
   * Create the next unique ID value which is unique in the reserved ID set.
   * 
   * The created ID is added to the set of reserved IDs.
   * 
   * @return The next ID value.
   */
  public String uniqueId() {
    String nextId;

    while (reserved.contains(nextId = nextId()));
    
    reserved.add(nextId);
    
    return nextId;
        
  }
  
}
