package at.gv.egovernment.moa.spss.tsl.config;

import iaik.util.logging.GeneralLog;
import iaik.xml.crypto.tsl.ex.TSLEngineDiedException;

import java.net.MalformedURLException;
import java.net.URL;

public class Configurator {

	private static final String _TMPDBFILENAME = "temp_tsl.sqlite";
	
	private static boolean _sqlMultithreaded;
	private static boolean _throwExceptions;
	private static boolean _logExceptions;
	private static boolean _throwWarnings;
	private static boolean _logWarnings;
	private static boolean _nullRedundancies;
	private static URL _euTSLURL;
	private static String _TSLWorkingDirectoryPath;
	private static String _dbFile;
	private static String _euTrustAnchorsPath;
	private static String _msTrustAnchorsPath;
	
	
	private static boolean _isInitialised = false;
	
	
	/**
	 * 
	 */
	public static void initial(String euTSLURL, String TSLWorkingDirectoryPath, String jdbcURL, String jdbcDriverClass) 
			throws TSLEngineDiedException {
		
		
		if (!_isInitialised) {	
			try {
				_euTSLURL = new URL(euTSLURL);
			} catch (MalformedURLException e) {
				GeneralLog.err("Bad TSL URL: " + euTSLURL, e);
				throw new TSLEngineDiedException(e);
			}
			
			if (!TSLWorkingDirectoryPath.endsWith("/"))
				TSLWorkingDirectoryPath += "/";
			
			Configurator._TSLWorkingDirectoryPath = TSLWorkingDirectoryPath;
			
			initialDefaultConfig();
			
			_isInitialised = true;
		}
	}
	
	public static String get_TSLWorkingDirectoryPath() {
		return _TSLWorkingDirectoryPath;
	}
	
	public static String get_dbFile() {
		return _dbFile;
	}

	public static void set_dbFileName(String _dbFile) {
		Configurator._dbFile = _TSLWorkingDirectoryPath + _dbFile;
	}
	
	public static String get_euTrustAnchorsPath() {
		return _euTrustAnchorsPath;
	}

	public static String get_msTrustAnchorsPath() {
		return _msTrustAnchorsPath;
	}

	public static boolean is_sqlMultithreaded() {
		return _sqlMultithreaded;
	}

	public static boolean is_throwExceptions() {
		return _throwExceptions;
	}

	public static boolean is_logExceptions() {
		return _logExceptions;
	}

	public static boolean is_throwWarnings() {
		return _throwWarnings;
	}

	public static boolean is_logWarnings() {
		return _logWarnings;
	}

	public static boolean is_nullRedundancies() {
		return _nullRedundancies;
	}

	public static URL get_euTSLURL() {
		return _euTSLURL;
	}

	public static boolean is_isInitialised() {
		return _isInitialised;
	}

	public static String get_TempdbFile() {
		return _TSLWorkingDirectoryPath + _TMPDBFILENAME;
	}
	
	public static void set_euTrustAnchorsPath(String _euTrustAnchorsPath) {
		Configurator._euTrustAnchorsPath = _euTrustAnchorsPath;
	}

	public static void set_msTrustAnchorsPath(String _msTrustAnchorsPath) {
		Configurator._msTrustAnchorsPath = _msTrustAnchorsPath;
	}
	
	private static void initialDefaultConfig() {		
		_sqlMultithreaded = false;
		_throwExceptions = true;
		_logExceptions = true;
		_throwWarnings = false;
		_logWarnings = true;
		_nullRedundancies = false;
		_dbFile = _TSLWorkingDirectoryPath + "tsl.sqlite";
		_euTrustAnchorsPath = _TSLWorkingDirectoryPath +  "trust/eu/";
		_msTrustAnchorsPath = "/trust/ms/";
	}	
}
