/*
 * Copyright 2003 Federal Chancellery Austria
 * MOA-SPSS has been developed in a cooperation between BRZ, the Federal
 * Chancellery Austria - ICT staff unit, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * http://www.osor.eu/eupl/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 */


package at.gv.egovernment.moa.spss.util;

import java.util.Locale;

import at.gv.egovernment.moaspss.util.Messages;

/**
 * Singleton wrapper around a <code>Messages</code> object.
 * 
 * @author Patrick Peck
 * @version $Id$
 */
public class MessageProvider {
  
  /** The resource names of the messages to load. */  
  private static final String[] DEFAULT_MESSAGE_RESOURCES =
    { "resources/properties/spss_messages" };
  /** The corresponding message locales. */
  private static final Locale[] DEFAULT_MESSAGE_LOCALES =
    new Locale[] { new Locale("de", "AT") };
  /** The single instance of this class. */
  private static MessageProvider instance;
  
  /** The messages provided by the <code>MessageProvider</code>. */
  private Messages messages;
  
  /**
   * Return the single instance of the <code>MessageProvider</code>.
   * 
   * Intialilizes the <code>MessageProvider</code> with the default message
   * locations: <code>/resources/properties/spss_messages</code>.
   * 
   * @return The single <code>MessageProvider</code>.
   */
  public static synchronized MessageProvider getInstance() {
    if (instance == null) {
      instance =
        new MessageProvider(DEFAULT_MESSAGE_RESOURCES, DEFAULT_MESSAGE_LOCALES);
    }
    return instance;
  }

  /**
   * Create a <code>MessageProvider</code>.
   * 
   * @param resourceNames The names of the resources containing the messages.
   * @param locales The corresponding locales.
   */
  protected MessageProvider(String[] resourceNames, Locale[] locales) {
    this.messages = new Messages(resourceNames, locales);
  }

  /**
   * Get the message corresponding to a given message ID.
   *
   * @param messageId The ID of the message.
   * @param parameters The parameters to fill in into the message arguments.
   * @return The formatted message. 
   */
  public String getMessage(String messageId, Object[] parameters) {
    return messages.getMessage(messageId, parameters);
  }

   
}
