/*
 * Copyright 2003 Federal Chancellery Austria
 * MOA-SPSS has been developed in a cooperation between BRZ, the Federal
 * Chancellery Austria - ICT staff unit, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * http://www.osor.eu/eupl/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 */


package test.at.gv.egovernment.moa.spss;

import java.security.Security;

import test.at.gv.egovernment.moa.MOATestCase;
import at.gv.egovernment.moaspss.logging.Logger;
import at.gv.egovernment.moaspss.logging.LoggingContext;
import at.gv.egovernment.moaspss.logging.LoggingContextManager;
import at.gv.egovernment.moaspss.util.MessageProvider;
import at.gv.egovernment.moa.spss.server.config.ConfigurationProvider;
import at.gv.egovernment.moa.spss.server.iaik.config.IaikConfigurator;
import at.gv.egovernment.moa.spss.server.transaction.TransactionContext;
import at.gv.egovernment.moa.spss.server.transaction.TransactionContextManager;

/**
 * Base class for MOA test cases.
 * 
 * Provides some utility functions.
 * 
 * @author Patrick Peck
 * @version $Id$
 */
public class SPSSTestCase extends MOATestCase {

  protected static final String TESTDATA_ROOT = "data/test/";

  /**
   * Constructor for MOATestCase.
   * @param arg0
   */
  public SPSSTestCase(String name) {
    super(name);
  }

  /**
   * Set up a transaction context with a test configuration.
   */
  protected void setUpTransactionContext() throws Exception {
    System.setProperty(
      ConfigurationProvider.CONFIG_PROPERTY_NAME,
      "data/test/conf/moa-spss/MOA-SPSSConfiguration.xml");
    ConfigurationProvider config = ConfigurationProvider.getInstance();
    TransactionContext context = new TransactionContext("test", null, config);
    TransactionContextManager.getInstance().setTransactionContext(context);
  }
  
  protected void setUpLoggingContext() throws Exception {
    LoggingContext context = new LoggingContext("test");
    LoggingContextManager.getInstance().setLoggingContext(context);
  }

  /**
   * Configure the IAIK modules with the current configuration.
   * 
   * A <code>TransactionContext</code> must have been set up before.
   */
  protected void setUpIaikConfiguration() throws Exception {
    TransactionContext context =
      TransactionContextManager.getInstance().getTransactionContext();
    ClassLoader cl = getClass().getClassLoader();
    MessageProvider msg = MessageProvider.getInstance();
    
    try {
      cl.loadClass("javax.security.cert.Certificate"); // from jcert.jar
    } catch (ClassNotFoundException e) {
      Logger.warn(msg.getMessage("init.03", null), e);
    }

    new IaikConfigurator().configure(context.getConfiguration());
  }
  
  protected void setUpSSL() throws Exception {
    //System.setProperty("javax.net.debug", "all");
    Security.addProvider(new com.sun.net.ssl.internal.ssl.Provider());
    System.setProperty(
      "java.protocol.handler.pkgs",
      "com.sun.net.ssl.internal.www.protocol");
  }

}
