/*
 * Copyright 2003 Federal Chancellery Austria
 * MOA-SPSS has been developed in a cooperation between BRZ, the Federal
 * Chancellery Austria - ICT staff unit, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * http://www.osor.eu/eupl/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 */


package test.at.gv.egovernment.moa.spss.api.xmlbind;

import org.w3c.dom.Element;

import test.at.gv.egovernment.moa.spss.SPSSTestCase;

import at.gv.egovernment.moa.spss.api.cmsverify.CMSContentExcplicit;
import at.gv.egovernment.moa.spss.api.cmsverify.VerifyCMSSignatureRequest;
import at.gv.egovernment.moa.spss.api.common.MetaInfo;
import at.gv.egovernment.moa.spss.api.xmlbind.VerifyCMSSignatureRequestParser;
import at.gv.egovernment.moaspss.util.DateTimeUtils;

/**
 * Test the <code>VerifyCMSSignatureRequestParserTest</code>.
 * 
 * @author Patrick Peck
 * @version $Id$
 */
public class VerifyCMSSignatureRequestParserTest extends SPSSTestCase {
  private static String TESTDATA_BASE =
    TESTDATA_ROOT + "xml/VerifyCMSSignature/";

  private VerifyCMSSignatureRequestParser requestParser;

  public VerifyCMSSignatureRequestParserTest(String name) {
    super(name);
  }

  protected void setUp() throws Exception {
    requestParser = new VerifyCMSSignatureRequestParser();
  }

  public void testParse() throws Exception {
    Element requestElem =
      parseXml(TESTDATA_BASE + "TestGeneratorVC0.001.Req.xml")
        .getDocumentElement();
    VerifyCMSSignatureRequest request = requestParser.parse(requestElem);
    MetaInfo metaInfo;
    CMSContentExcplicit content;

    assertNotNull(request);
    assertEquals(1, request.getSignatories()[0]);
    assertEquals(
      DateTimeUtils.parseDateTime("2003-04-04T09:30:47-05:00"),
      request.getDateTime());
    assertNotNull(request.getCMSSignature());
    assertNotNull(request.getDataObject());
    assertEquals("TrustProfile1", request.getTrustProfileId());
    
    metaInfo = request.getDataObject().getMetaInfo();
    assertNotNull(metaInfo);
    assertEquals("text/plain", metaInfo.getMimeType());
    assertEquals("http://10.16.46.109/TestDatenGenerator/resources/testDaten.txt", metaInfo.getDescription());
    
    content = (CMSContentExcplicit) request.getDataObject().getContent();
    assertNotNull(content.getBinaryContent());
    
  }

}
