/*
 * Copyright 2003 Federal Chancellery Austria
 * MOA-SPSS has been developed in a cooperation between BRZ, the Federal
 * Chancellery Austria - ICT staff unit, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * http://www.osor.eu/eupl/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 */


package test.at.gv.egovernment.moa.spss.server.invoke;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import test.at.gv.egovernment.moa.spss.SPSSTestCase;
import at.gv.egovernment.moa.spss.api.cmsverify.VerifyCMSSignatureRequest;
import at.gv.egovernment.moa.spss.api.cmsverify.VerifyCMSSignatureResponse;
import at.gv.egovernment.moa.spss.api.xmlbind.VerifyCMSSignatureRequestParser;
import at.gv.egovernment.moa.spss.api.xmlbind.VerifyCMSSignatureResponseBuilder;
import at.gv.egovernment.moa.spss.server.invoke.CMSSignatureVerificationInvoker;
import at.gv.egovernment.moaspss.util.DOMUtils;

/**
 * Mainly a smoke test for debugging the CMSSignatureVerificationInvoker.
 * 
 * @author Patrick Peck
 * @version $Id$
 */
public class CMSSignatureVerificationInvokerTest extends SPSSTestCase {
  private static final String TESTDATA_BASE =
    TESTDATA_ROOT + "xml/VerifyCMSSignature/";

  /**
   * Constructor for CMSSignatureVerificationInvokerTest.
   * @param name
   */
  public CMSSignatureVerificationInvokerTest(String name) {
    super(name);
  }

  protected void setUp() throws Exception {
    setUpTransactionContext();
    setUpLoggingContext();
    setUpIaikConfiguration();
  }

  public void testVerifyCMSSignature() throws Exception {
    try {
      CMSSignatureVerificationInvoker invoker =
        CMSSignatureVerificationInvoker.getInstance();
      VerifyCMSSignatureRequestParser requestParser =
        new VerifyCMSSignatureRequestParser();
      Document doc =
        SPSSTestCase.parseXmlValidating(
          TESTDATA_BASE + "TestGeneratorVC0.001.Req.xml");
      VerifyCMSSignatureRequest request =
        requestParser.parse(doc.getDocumentElement());
      VerifyCMSSignatureResponse response = invoker.verifyCMSSignature(request);
      VerifyCMSSignatureResponseBuilder responseBuilder =
        new VerifyCMSSignatureResponseBuilder();
      Element result = responseBuilder.build(response).getDocumentElement();

      System.out.println(DOMUtils.serializeNode(result));
    } catch (Exception e) {
      e.printStackTrace();
      fail();
    }
  }

}
