/*
 * Copyright 2003 Federal Chancellery Austria
 * MOA-SPSS has been developed in a cooperation between BRZ, the Federal
 * Chancellery Austria - ICT staff unit, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * http://www.osor.eu/eupl/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 */


package test.at.gv.egovernment.moa.spss.server.invoke;

import java.util.Collections;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import test.at.gv.egovernment.moa.spss.SPSSTestCase;
import at.gv.egovernment.moa.spss.api.xmlbind.CreateXMLSignatureRequestParser;
import at.gv.egovernment.moa.spss.api.xmlbind.CreateXMLSignatureResponseBuilder;
import at.gv.egovernment.moa.spss.api.xmlsign.CreateXMLSignatureRequest;
import at.gv.egovernment.moa.spss.api.xmlsign.CreateXMLSignatureResponse;
import at.gv.egovernment.moa.spss.server.invoke.XMLSignatureCreationInvoker;
import at.gv.egovernment.moaspss.util.DOMUtils;

/**
 * Mainly a smoke test for debugging the XMLSignatureCreationInvoker.
 * 
 * @author Patrick Peck
 * @version $Id$
 */
public class XMLSignatureCreationInvokerTest extends SPSSTestCase {
  private static final String TESTDATA_BASE =
    TESTDATA_ROOT + "xml/CreateXMLSignature/";

  public XMLSignatureCreationInvokerTest(String name) {
    super(name);
  }

  protected void setUp() throws Exception {
    setUpTransactionContext();
    setUpLoggingContext();
    setUpIaikConfiguration();
    setUpSSL();
  }

  public void testCreateXMLSignature() throws Exception {
    try {
      XMLSignatureCreationInvoker invoker =
        XMLSignatureCreationInvoker.getInstance();
      CreateXMLSignatureRequestParser requestParser =
        new CreateXMLSignatureRequestParser();
      Document doc =
        SPSSTestCase.parseXmlValidating(
          TESTDATA_BASE + "TestGeneratorCX2.004.Req.xml");
      CreateXMLSignatureRequest request =
        requestParser.parse(doc.getDocumentElement());
      CreateXMLSignatureResponse response = 
        invoker.createXMLSignature(request, Collections.EMPTY_SET);
      CreateXMLSignatureResponseBuilder responseBuilder = 
        new CreateXMLSignatureResponseBuilder();
      Element result = responseBuilder.build(response).getDocumentElement();

      System.out.println(DOMUtils.serializeNode(result));
    } catch (Exception e) {
      e.printStackTrace();
      fail();
    }
  }

}
