/*
 * Copyright 2003 Federal Chancellery Austria
 * MOA-SPSS has been developed in a cooperation between BRZ, the Federal
 * Chancellery Austria - ICT staff unit, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * http://www.osor.eu/eupl/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 */


package at.gv.egovernment.moa.spss.server.service;

import java.io.ByteArrayInputStream;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import at.gv.egovernment.moa.spss.MOAApplicationException;
import at.gv.egovernment.moa.spss.util.MOASPSSEntityResolver;
import at.gv.egovernment.moaspss.util.Constants;
import at.gv.egovernment.moaspss.util.DOMUtils;
import at.gv.egovernment.moaspss.util.MOAErrorHandler;

/**
 * Helper methods for the Service classes.
 * 
 * @author Patrick Peck
 * @version $Id$
 */
public class ServiceUtils {

  /**
   * Schema-validate a request.
   * 
   * @param request The request to validate.
   * @throws MOAApplicationException An error occurred validating the requst.
   */
  public static void validateRequest(Element[] request)
    throws MOAApplicationException {

    // validate the request
    try {
      DOMUtils.validateElement(
        request[0],
        Constants.ALL_SCHEMA_LOCATIONS,
        null,
        new MOASPSSEntityResolver());
    } catch (Exception e) {
      throw new MOAApplicationException(
        "1100",
        new Object[] { e.getMessage()},
        e);
    }
  }

  /**
   * Reparse the request with schema-validation turned on so that ID references
   * are resolved.
   * 
   * @param request The request to reparse.
   * @return The reparsed request.
   * @throws MOAApplicationException An error occurred parsing the request.
   */
  public static Element reparseRequest(Element request)
    throws MOAApplicationException {

    try {
      byte[] requestBytes = DOMUtils.serializeNode(request, "UTF-8");
      Document validatedRequest = DOMUtils.parseDocument(new ByteArrayInputStream(requestBytes),
    	          true,
    	          Constants.ALL_SCHEMA_LOCATIONS,
    	          null,
    		      new MOASPSSEntityResolver(),
    		      new MOAErrorHandler());
    	  
//        DOMUtils.parseDocument(
//          new ByteArrayInputStream(requestBytes),
//          true,
//          Constants.ALL_SCHEMA_LOCATIONS,
//          null);
      return validatedRequest.getDocumentElement();
    } catch (Exception e) {
      throw new MOAApplicationException(
        "1100",
        new Object[] { e.getMessage()},
        e);
    }
  }

}
